﻿using System;
using System.Windows;
using System.Windows.Controls;
using CWSWrapper;

namespace CWSTransApp
{
    /// <summary>
    /// Interaction logic for GeneralSettings.xaml
    /// </summary>
    public partial class GeneralSettings : Window
    {
        private String m_TerminalDeviceType;
        private String m_TerminalCatLevelIndicator;
        private String m_TerminalCardholderInputMode;
        private Boolean m_DoSetTerminalDeviceConfiguration;
        private Boolean m_DoSetTerminalCardholderInputModeConfiguration;
        private Boolean m_DoLanguageOverride;
        private Boolean m_DoDebitNetworkOverride;
        private Boolean m_DoCvmOptionsOverride;
        private Boolean m_DoSetLaneNumber;
        private String m_LanguageCode;
        private String m_CountryCode;
        private String m_VisaPreference;
        private String m_MastercardPreference;
        private String m_DiscoverPreference;
        private String m_SignatureOption;
        private String m_SignatureOptionDefault = SignatureOption.DISABLED;
        private String m_LaneNumber;
        private PropertyFile propertyFile;

        public GeneralSettings()
        {
            InitializeComponent();
            loadControls();
        }

        private void loadControls()
        {
            propertyFile = MainWindow.LoadPropertyFile();

            string temp;
            cb_LanguageCode.Items.Add(LanguageCode.UNSET);
            cb_LanguageCode.Items.Add(LanguageCode.UNKNOWN);
            cb_LanguageCode.Items.Add(LanguageCode.ENGLISH);
            cb_LanguageCode.Items.Add(LanguageCode.FRENCH);
            cb_LanguageCode.Items.Add(LanguageCode.SPANISH);
            cb_LanguageCode.SelectedIndex = 0;
            if (null != (temp = propertyFile.get("overrideLanguageLanguageCode")))
            {
                if (temp == "EN")
                {
                    cb_LanguageCode.SelectedIndex = cb_LanguageCode.Items.IndexOf(LanguageCode.ENGLISH);
                }
                else if (temp == "FR")
                {
                    cb_LanguageCode.SelectedIndex = cb_LanguageCode.Items.IndexOf(LanguageCode.FRENCH);
                }
                else if (temp == "ES")
                {
                    cb_LanguageCode.SelectedIndex = cb_LanguageCode.Items.IndexOf(LanguageCode.SPANISH);
                }
                else if (temp == "UNKNOWN")
                {
                    cb_LanguageCode.SelectedIndex = cb_LanguageCode.Items.IndexOf(LanguageCode.UNKNOWN);
                }
            }
            
            cb_CountryCode.Items.Add(CountryCode.UNSET);
            cb_CountryCode.Items.Add(CountryCode.UNKNOWN);
            cb_CountryCode.Items.Add(CountryCode.US);
            cb_CountryCode.Items.Add(CountryCode.CA);
            cb_CountryCode.SelectedIndex = 0;
            if (null != (temp = propertyFile.get("overrideLanguageCountryCode")))
            {
                if (temp == "US")
                {
                    cb_CountryCode.SelectedIndex = cb_CountryCode.Items.IndexOf(CountryCode.US);
                }
                else if (temp == "CA")
                {
                    cb_CountryCode.SelectedIndex = cb_CountryCode.Items.IndexOf(CountryCode.CA);
                }
                else if (temp == "UNKNOWN")
                {
                    cb_CountryCode.SelectedIndex = cb_CountryCode.Items.IndexOf(CountryCode.UNKNOWN);
                }
            }

            setupTerminalDeviceCtrls();

            setupTerminalCardholderInputModeCtrls();

            setUpDebitNetworkCtrls(visaCheckBox, cb_VisaDebitSetting, VisaCheckBox_Checked, "visaDebitNetworkPreference");
            setUpDebitNetworkCtrls(mastercardCheckBox, cb_MastercardDebitSetting, MastercardCheckBox_Checked, "mastercardDebitNetworkPreference");
            setUpDebitNetworkCtrls(discoverCheckBox, cb_DiscoverDebitSetting, DiscoverCheckBox_Checked, "discoverDebitNetworkPreference");

            FillSignatureOption();
            setUpLaneNumber();
        }

        private void setUpLaneNumber()
        {
            laneNumberCheckBox.IsChecked = false;
            laneNumberCheckBox.Checked += LaneNumberCheckBox_Checked;
            laneNumberCheckBox.Unchecked += LaneNumberCheckBox_Checked;

            String laneNumber;
            if (null != (laneNumber = propertyFile.get("laneNumber")))
            {
                laneNumberCheckBox.IsChecked = true;
                tb_LaneNumber.Text = laneNumber;
            }

            tb_LaneNumber.IsEnabled = (laneNumberCheckBox.IsChecked ?? false) ? true : false;
        }

        private void FillSignatureOption()
        {
            signatureOptionCheckBox.IsChecked = false;
            signatureOptionCheckBox.Checked += SignatureOptionCheckBox_Checked;
            signatureOptionCheckBox.Unchecked += SignatureOptionCheckBox_Checked;

            String signatureOption;
            if (null == (signatureOption = propertyFile.get("signatureOption")))
            {
                signatureOption = m_SignatureOptionDefault;
            }
            else
            {
                signatureOptionCheckBox.IsChecked = true;
            }

            signatureOptionComboBox.Items.Add(SignatureOption.DISABLED);
            signatureOptionComboBox.Items.Add(SignatureOption.OPTIONAL);
            signatureOptionComboBox.Items.Add(SignatureOption.ENABLED_MSR_ONLY);
            signatureOptionComboBox.Items.Add(SignatureOption.ENABLED_ALL);

            signatureOptionComboBox.SelectedIndex = signatureOptionComboBox.Items.IndexOf(signatureOption);
            signatureOptionComboBox.IsEnabled = (signatureOptionCheckBox.IsChecked ?? false) ? true : false;
        }

        private void setupTerminalDeviceCtrls()
        {
            string temp;

            cb_TerminalDeviceType.Items.Add(TerminalDeviceType.UNSET);
            cb_TerminalDeviceType.Items.Add(TerminalDeviceType.UNKNOWN);
            cb_TerminalDeviceType.Items.Add(TerminalDeviceType.ATTENDED_TERMINAL);
            cb_TerminalDeviceType.Items.Add(TerminalDeviceType.LIMITED_AMOUNT_TERMINAL);
            cb_TerminalDeviceType.Items.Add(TerminalDeviceType.UNATTENDED_TERMINAL_ATM);
            cb_TerminalDeviceType.Items.Add(TerminalDeviceType.UNATTENDED_TERMINAL_CAT);
            cb_TerminalDeviceType.Items.Add(TerminalDeviceType.NO_TERMINAL);
            cb_TerminalDeviceType.Items.Add(TerminalDeviceType.MPOS);
            cb_TerminalDeviceType.Items.Add(TerminalDeviceType.UNRECOGNIZED);

            cb_TerminalDeviceType.SelectedIndex = 0;
            if (null != (temp = propertyFile.get("terminalDeviceType")))
            {
                try
                {
                    TerminalDeviceType deviceType = (TerminalDeviceType)Enum.Parse(typeof(TerminalDeviceType), temp);

                    switch (deviceType)
                    {
                        case TerminalDeviceType.UNKNOWN:
                            cb_TerminalDeviceType.SelectedIndex = cb_TerminalDeviceType.Items.IndexOf(TerminalDeviceType.UNKNOWN);
                            break;
                        case TerminalDeviceType.ATTENDED_TERMINAL:
                            cb_TerminalDeviceType.SelectedIndex = cb_TerminalDeviceType.Items.IndexOf(TerminalDeviceType.ATTENDED_TERMINAL);
                            break;
                        case TerminalDeviceType.LIMITED_AMOUNT_TERMINAL:
                            cb_TerminalDeviceType.SelectedIndex = cb_TerminalDeviceType.Items.IndexOf(TerminalDeviceType.LIMITED_AMOUNT_TERMINAL);
                            break;
                        case TerminalDeviceType.UNATTENDED_TERMINAL_ATM:
                            cb_TerminalDeviceType.SelectedIndex = cb_TerminalDeviceType.Items.IndexOf(TerminalDeviceType.UNATTENDED_TERMINAL_ATM);
                            break;
                        case TerminalDeviceType.UNATTENDED_TERMINAL_CAT:
                            cb_TerminalDeviceType.SelectedIndex = cb_TerminalDeviceType.Items.IndexOf(TerminalDeviceType.UNATTENDED_TERMINAL_CAT);
                            break;
                        case TerminalDeviceType.NO_TERMINAL:
                            cb_TerminalDeviceType.SelectedIndex = cb_TerminalDeviceType.Items.IndexOf(TerminalDeviceType.NO_TERMINAL);
                            break;
                        case TerminalDeviceType.MPOS:
                            cb_TerminalDeviceType.SelectedIndex = cb_TerminalDeviceType.Items.IndexOf(TerminalDeviceType.MPOS);
                            break;
                        case TerminalDeviceType.UNRECOGNIZED:
                            cb_TerminalDeviceType.SelectedIndex = cb_TerminalDeviceType.Items.IndexOf(TerminalDeviceType.UNRECOGNIZED);
                            break;
                    }
                }
                catch (ArgumentException ex)
                {
                    cb_TerminalDeviceType.SelectedIndex = cb_TerminalDeviceType.Items.IndexOf(TerminalDeviceType.UNKNOWN);
                }
            }
            else
            {
                cb_TerminalDeviceType.SelectedIndex = cb_TerminalDeviceType.Items.IndexOf(TerminalDeviceType.UNSET);
            }

            cb_TerminalCatLevelIndicator.Items.Add(CatLevelIndicator.UNSET);
            cb_TerminalCatLevelIndicator.Items.Add(CatLevelIndicator.UNKNOWN);
            cb_TerminalCatLevelIndicator.Items.Add(CatLevelIndicator.TRANSACTION_NOT_APPLICABLE);
            cb_TerminalCatLevelIndicator.Items.Add(CatLevelIndicator.AUTOMATED_DISPENSING_MACHINE);
            cb_TerminalCatLevelIndicator.Items.Add(CatLevelIndicator.SELF_SERVICE_TERMINAL);
            cb_TerminalCatLevelIndicator.Items.Add(CatLevelIndicator.LIMITED_AMOUNT_TERMINAL);
            cb_TerminalCatLevelIndicator.Items.Add(CatLevelIndicator.IN_FLIGHT_TERMINAL);
            cb_TerminalCatLevelIndicator.Items.Add(CatLevelIndicator.TRANSPONDER);
            cb_TerminalCatLevelIndicator.Items.Add(CatLevelIndicator.MPOS);

            cb_TerminalCatLevelIndicator.SelectedIndex = 0;
            if (null != (temp = propertyFile.get("terminalCatLevelIndicator")))
            { 
                try
                {
                    CatLevelIndicator catLevelIndicator = (CatLevelIndicator)Enum.Parse(typeof(CatLevelIndicator), temp);

                    switch (catLevelIndicator)
                    {
                        case CatLevelIndicator.UNKNOWN:
                            cb_TerminalCatLevelIndicator.SelectedIndex = cb_TerminalCatLevelIndicator.Items.IndexOf(CatLevelIndicator.UNKNOWN);
                            break;
                        case CatLevelIndicator.TRANSACTION_NOT_APPLICABLE:
                            cb_TerminalCatLevelIndicator.SelectedIndex = cb_TerminalCatLevelIndicator.Items.IndexOf(CatLevelIndicator.TRANSACTION_NOT_APPLICABLE);
                            break;
                        case CatLevelIndicator.AUTOMATED_DISPENSING_MACHINE:
                            cb_TerminalCatLevelIndicator.SelectedIndex = cb_TerminalCatLevelIndicator.Items.IndexOf(CatLevelIndicator.AUTOMATED_DISPENSING_MACHINE);
                            break;
                        case CatLevelIndicator.SELF_SERVICE_TERMINAL:
                            cb_TerminalCatLevelIndicator.SelectedIndex = cb_TerminalCatLevelIndicator.Items.IndexOf(CatLevelIndicator.SELF_SERVICE_TERMINAL);
                            break;
                        case CatLevelIndicator.LIMITED_AMOUNT_TERMINAL:
                            cb_TerminalCatLevelIndicator.SelectedIndex = cb_TerminalCatLevelIndicator.Items.IndexOf(CatLevelIndicator.LIMITED_AMOUNT_TERMINAL);
                            break;
                        case CatLevelIndicator.IN_FLIGHT_TERMINAL:
                            cb_TerminalCatLevelIndicator.SelectedIndex = cb_TerminalCatLevelIndicator.Items.IndexOf(CatLevelIndicator.IN_FLIGHT_TERMINAL);
                            break;
                        case CatLevelIndicator.TRANSPONDER:
                            cb_TerminalCatLevelIndicator.SelectedIndex = cb_TerminalCatLevelIndicator.Items.IndexOf(CatLevelIndicator.TRANSPONDER);
                            break;
                        case CatLevelIndicator.MPOS:
                            cb_TerminalCatLevelIndicator.SelectedIndex = cb_TerminalCatLevelIndicator.Items.IndexOf(CatLevelIndicator.MPOS);
                            break;
                    }
                }
                catch (ArgumentException ex)
                {
                    cb_TerminalCatLevelIndicator.SelectedIndex = cb_TerminalCatLevelIndicator.Items.IndexOf(CatLevelIndicator.UNKNOWN);
                }
            }
            else
            {
                cb_TerminalCatLevelIndicator.SelectedIndex = cb_TerminalCatLevelIndicator.Items.IndexOf(CatLevelIndicator.UNSET);
            }
        }

        private void setupTerminalCardholderInputModeCtrls()
        {
            string temp;

            cb_TerminalCardholderInputMode.Items.Add(TerminalCardholderInputMode.UNSET);
            cb_TerminalCardholderInputMode.Items.Add(TerminalCardholderInputMode.UNKNOWN);
            cb_TerminalCardholderInputMode.Items.Add(TerminalCardholderInputMode.IN_POS);
            cb_TerminalCardholderInputMode.Items.Add(TerminalCardholderInputMode.CARD_READER);

            cb_TerminalCardholderInputMode.SelectedIndex = 0;
            if (null != (temp = propertyFile.get("terminalCardholderInputMode")))
            {
                try
                {
                    TerminalCardholderInputMode terminalCardholderInputMode = (TerminalCardholderInputMode)Enum.Parse(typeof(TerminalCardholderInputMode), temp);

                    switch (terminalCardholderInputMode)
                    {
                        case TerminalCardholderInputMode.UNKNOWN:
                            cb_TerminalCardholderInputMode.SelectedIndex = cb_TerminalCardholderInputMode.Items.IndexOf(TerminalCardholderInputMode.UNKNOWN);
                            break;
                        case TerminalCardholderInputMode.IN_POS:
                            cb_TerminalCardholderInputMode.SelectedIndex = cb_TerminalCardholderInputMode.Items.IndexOf(TerminalCardholderInputMode.IN_POS);
                            break;
                        case TerminalCardholderInputMode.CARD_READER:
                            cb_TerminalCardholderInputMode.SelectedIndex = cb_TerminalCardholderInputMode.Items.IndexOf(TerminalCardholderInputMode.CARD_READER);
                            break;
                    }
                }
                catch (ArgumentException ex)
                {
                    cb_TerminalCardholderInputMode.SelectedIndex = cb_TerminalCardholderInputMode.Items.IndexOf(TerminalCardholderInputMode.UNKNOWN);
                }
            }
            else
            {
                cb_TerminalCardholderInputMode.SelectedIndex = cb_TerminalCardholderInputMode.Items.IndexOf(TerminalCardholderInputMode.UNSET);
            }
        }

        private void setUpDebitNetworkCtrls(CheckBox checkBox, ComboBox comboBox, RoutedEventHandler checkedEvent, string propertyName)
        {
            checkBox.IsChecked = false;
            checkBox.Checked += checkedEvent;
            checkBox.Unchecked += checkedEvent;
            comboBox.Items.Add(DebitNetworkChoice.NO_PREFERENCE);
            comboBox.Items.Add(DebitNetworkChoice.US_COMMON_DEBIT);
            comboBox.Items.Add(DebitNetworkChoice.GLOBAL_NETWORK);
            comboBox.SelectedIndex = 0;
            comboBox.Visibility = Visibility.Hidden;
            string temp;
            if (null != (temp = propertyFile.get(propertyName)))
            {
                checkBox.IsChecked = true;
                comboBox.Visibility = Visibility.Visible;
                if (temp == "US_COMMON_DEBIT")
                {
                    comboBox.SelectedIndex = comboBox.Items.IndexOf(DebitNetworkChoice.US_COMMON_DEBIT);
                }
                else if (temp == "GLOBAL_NETWORK")
                {
                    comboBox.SelectedIndex = comboBox.Items.IndexOf(DebitNetworkChoice.GLOBAL_NETWORK);
                }
            }
        }

        private void VisaCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            cb_VisaDebitSetting.Visibility = (visaCheckBox.IsChecked ?? false) ? Visibility.Visible : Visibility.Hidden;
        }

        private void MastercardCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            cb_MastercardDebitSetting.Visibility = (mastercardCheckBox.IsChecked ?? false) ? Visibility.Visible : Visibility.Hidden;
        }

        private void DiscoverCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            cb_DiscoverDebitSetting.Visibility = (discoverCheckBox.IsChecked ?? false) ? Visibility.Visible : Visibility.Hidden;
        }

        private void SignatureOptionCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            signatureOptionComboBox.IsEnabled = (signatureOptionCheckBox.IsChecked ?? false) ? true : false;
        }

        private void LaneNumberCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            tb_LaneNumber.IsEnabled = (laneNumberCheckBox.IsChecked ?? false) ? true : false;
        }

        private void Set_TerminalDevice_Click(object sender, RoutedEventArgs e)
        {
            m_TerminalDeviceType = cb_TerminalDeviceType.SelectedItem.ToString();
            m_TerminalCatLevelIndicator = cb_TerminalCatLevelIndicator.SelectedItem.ToString();

            propertyFile.set("terminalDeviceType", m_TerminalDeviceType);
            propertyFile.set("terminalCatLevelIndicator", m_TerminalCatLevelIndicator);
            propertyFile.Save();

            DialogResult = true;
            m_DoSetTerminalDeviceConfiguration = true;
            this.Close();
        }

        private void Set_TerminalCardholderInputMode_Click(object sender, RoutedEventArgs e)
        {
            m_TerminalCardholderInputMode = cb_TerminalCardholderInputMode.SelectedItem.ToString();

            propertyFile.set("terminalCardholderInputMode", m_TerminalCardholderInputMode);
            propertyFile.Save();

            DialogResult = true;
            m_DoSetTerminalCardholderInputModeConfiguration = true;
            this.Close();
        }

        private void Override_Language_Click(object sender, RoutedEventArgs e)
        {
            m_LanguageCode = cb_LanguageCode.SelectedItem.ToString();
            m_CountryCode = cb_CountryCode.SelectedItem.ToString();

            propertyFile.set("overrideLanguageLanguageCode", m_LanguageCode);
            propertyFile.set("overrideLanguageCountryCode", m_CountryCode);
            propertyFile.Save();

            DialogResult = true;
            m_DoLanguageOverride = true;
            this.Close();
        }

        private void Override_DebitNetwork_Click(object sender, RoutedEventArgs e)
        {
            m_VisaPreference = null;
            m_MastercardPreference = null;
            m_DiscoverPreference = null;

            if (visaCheckBox.IsChecked.Value)
            {
                m_VisaPreference = cb_VisaDebitSetting.SelectedItem.ToString();
                propertyFile.set("visaDebitNetworkPreference", DebitNetworkChoice.toApiValue(m_VisaPreference));
            }
            else
            {
                propertyFile.remove("visaDebitNetworkPreference");
            }

            if (mastercardCheckBox.IsChecked.Value)
            {
                m_MastercardPreference = cb_MastercardDebitSetting.SelectedItem.ToString();
                propertyFile.set("mastercardDebitNetworkPreference", DebitNetworkChoice.toApiValue(m_MastercardPreference));
            }
            else
            {
                propertyFile.remove("mastercardDebitNetworkPreference");
            }

            if (discoverCheckBox.IsChecked.Value)
            {
                m_DiscoverPreference = cb_DiscoverDebitSetting.SelectedItem.ToString();
                propertyFile.set("discoverDebitNetworkPreference", DebitNetworkChoice.toApiValue(m_DiscoverPreference));
            }
            else
            {
                propertyFile.remove("discoverDebitNetworkPreference");
            }
            
            propertyFile.Save();

            DialogResult = true;
            m_DoDebitNetworkOverride = true;
            this.Close();
        }

        private void Override_CvmOptions_Click(object sender, RoutedEventArgs e)
        {
            if (signatureOptionCheckBox.IsChecked.Value)
            {
                m_SignatureOption = signatureOptionComboBox.SelectedItem.ToString();
                propertyFile.set("signatureOption", m_SignatureOption);
            }
            else
            {
                propertyFile.remove("signatureOption");
            }

            propertyFile.Save();

            DialogResult = true;
            m_DoCvmOptionsOverride = true;
            this.Close();
        }

        private void Cancel_Click(object sender, RoutedEventArgs e)
        {
            m_DoSetTerminalDeviceConfiguration = false;
            m_DoSetTerminalCardholderInputModeConfiguration = false;
            m_DoLanguageOverride = false;
            m_DoDebitNetworkOverride = false;
            m_DoCvmOptionsOverride = false;
            m_DoSetLaneNumber = false;
            DialogResult = false;
            this.Close();
        }

        public Boolean DoSetTerminalDeviceConfiguration()
        {
            return m_DoSetTerminalDeviceConfiguration;
        }

        public Boolean DoSetTerminalCardholderInputModeConfiguration()
        {
            return m_DoSetTerminalCardholderInputModeConfiguration;
        }

        public Boolean DoLanguageOverride()
        {
            return m_DoLanguageOverride;
        }

        public Boolean DoDebitNetworkOverride()
        {
            return m_DoDebitNetworkOverride;
        }

        public Boolean DoCvmOptionsOverride()
        {
            return m_DoCvmOptionsOverride;
        }

        public Boolean DoSetLaneNumber()
        {
            return m_DoSetLaneNumber;
        }

        public String getTerminalDeviceType()
        {
            return m_TerminalDeviceType;
        }

        public String getTerminalCatLevelIndicator()
        {
            return m_TerminalCatLevelIndicator;
        }

        public String getTerminalCardholderInputMode()
        {
            return m_TerminalCardholderInputMode;
        }

        public String getLanguageCode()
        {
            return m_LanguageCode;
        }

        public String getCountryCode()
        {
            return m_CountryCode;
        }

        public String getVisaDebitNetworkPreference()
        {
            return m_VisaPreference;
        }

        public String getMastercardDebitNetworkPreference()
        {
            return m_MastercardPreference;
        }

        public String getDiscoverDebitNetworkPreference()
        {
            return m_DiscoverPreference;
        }

        public String getSignatureOption()
        {
            return m_SignatureOption;
        }

        public String getLaneNumber()
        {
            return m_LaneNumber;
        }

        private void Set_LaneNumber_Click(object sender, RoutedEventArgs e)
        {
            if (laneNumberCheckBox.IsChecked.Value)
            {
                m_LaneNumber = tb_LaneNumber.Text;
                propertyFile.set("laneNumber", m_LaneNumber);
            }
            else
            {
                m_LaneNumber = null;
                propertyFile.remove("laneNumber");
            }

            propertyFile.Save();

            DialogResult = true;
            m_DoSetLaneNumber = true;
            this.Close();
        }
    }
}
