﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Threading;
using System.IO;

using CWSWrapper;
using System.Globalization;

namespace CWSTransApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
#pragma warning disable 414     // Suppress warning about member set but not used

        /* NOTE:  These are fallback values.  Credential are loaded from CWSTransApp.properties.  See OpenPaymentGateway() */

        private static readonly String MY_SERVERTYPE = "DEMO";
        private static readonly String MY_EMAIL = "hello@there.com";
        private static readonly String MY_MERCHANT = "123456";
        private static readonly String MY_USER = "somedude";
        private static readonly String MY_PIN = "ABCD";
        private static readonly String MY_VENDOR_ID = "myvendorid";
        private static readonly String MY_VENDOR_APP_NAME = "myvendorappname";
        private static readonly String MY_VENDOR_APP_VERSION = "myvendorappversion";
        private static readonly TerminalDeviceType MY_TERMINAL_DEVICE_TYPE = TerminalDeviceType.UNSET;
        private static readonly CatLevelIndicator MY_TERMINAL_CAT_LEVEL_INDICATOR = CatLevelIndicator.UNSET;
        private static readonly TerminalCardholderInputMode MY_TERMINAL_CARDHOLDER_INPUT_MODE = TerminalCardholderInputMode.UNSET;
        private static readonly String MY_OVERRIDE_LANGUAGE_LANGUAGE_CODE = "UNSET";
        private static readonly String MY_OVERRIDE_LANGUAGE_COUNTRY_CODE = "UNSET";

        private static readonly Decimal dZero = new Decimal(0);

        CWS m_CWS;
        private string m_PaymentGatewayId;
        private string m_OriginalTransId = null;
        private List<PaymentTransactionData> m_RecentTransactionOutcomes;

        private string m_ChanId;
        private bool m_IsAsync;
        private PrintPaymentArgs m_PrintPaymentArgs;
        private PaymentTransactionData m_TransData;
        private QuickSelection m_QuickSelection;
        private AccountInfo m_AccountInfo;

        private int m_logcnt = 0;

        private enum TransState
        {
            Idle,
            InProgress,
            Paused
        }
        private TransState m_TransState;

        public MainWindow()
        {
            InitializeComponent();

            /* Initlalize our UI.  Load combo boxes and fill in default values. */

            FillTranType();
            FillTenderType();
            FillEntryType();
            FillCardType();
            FillCurrency();
            InitRecentTransactionOutcomes();
            FillRecentTransactionOutcomes();
            Random r = new Random();
            tb_Amount.Text = r.Next(1199, 8999).ToString();

            tb_Address.Text = "8985 Ridgeestone Ct";
            tb_Zip.Text = "30076";
            tb_FirstName.Text = "John";
            tb_LastName.Text = "Doe";
            tb_InvoiceNum.Text = "123";
            tb_LaneNumber.Text = "12345678";
            tb_Voice.Text = "321";
            tb_Tax.Text = "200";
            tb_Tip.Text = "300";
            tb_Discount.Text = "100";
            cb_Async.SetCurrentValue(CheckBox.IsCheckedProperty, true);
            m_IsAsync = true;
            tb_Signature.Text = "pete.jpg";
            tb_EmailAddress.Text = "joe.customer@e.com";
            //l_JsonResp.Content = "json";
            cb_Tax.SetCurrentValue(System.Windows.Controls.CheckBox.IsCheckedProperty, false);
            cb_Tip.SetCurrentValue(System.Windows.Controls.CheckBox.IsCheckedProperty, false);
            cb_Discount.SetCurrentValue(System.Windows.Controls.CheckBox.IsCheckedProperty, false);

            FillSignatureOption();

            /* Create an instance of CWS, then start it. */
            /* NOTE: adjust path to log file. */

            m_CWS = new CWS(); 
            m_CWS.Start("c:\\code\\cwswrapper.log", 4);
            Thread.Sleep(100);

            /* Not strictly necessary */

            m_CWS.DelayMillis = 1000;               /* one second between polls */
            // m_CWS.MaxWaitMillis = 2 * 60 * 1000;    /* two minutes */
            // m_CWS.MaxWaitMillis = -1;               /* No timeout */

            OpenPaymentGateway();       /* Send in our credentials */

            InitDeviceConnectionConfig();    /* Init connection method */


            InitPrintPaymentArgs();
            InitQuickSelection();

            SetTransState(TransState.Idle);
        }

        public static PropertyFile LoadPropertyFile()
        {
            /* Expect the properites file to be in the same directory are our EXE */

            string path = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location) + "\\CWSTransApp.properties";
            return new PropertyFile(path);
        }

        /*
         * OpenPaymentGateway - load up a OpenPaymentArgs object with our credential then
         * call CWS.OpenPaymentGateway.
         * This will return an ID that identifies us to CWS.  It actually passes these credentials
         * to Converge.
         * If something is wrong with the credentials there is no notificaion at this
         * time.  When you attempt a transaction, or validate account, you will get an
         * error about invalid credentials.
         */
        private void OpenPaymentGateway()
        {
            PropertyFile pf = LoadPropertyFile();           /* Load up the properties file */
            OpenPaymentArgs obea = new OpenPaymentArgs();
            obea.overrideDefaultTerminalLanguage = new OverrideDefaultTerminalLanguage();
            obea.overrideDefaultTerminalLanguage.languageInformation = new LanguageInformation();
            string temp;
            if (null == (temp = pf.get("merchant")))        /* Does it have a merchant? */
                obea.merchantId = MY_MERCHANT;              /* No, use fallback */
            else
                obea.merchantId = temp;

            if (null == (temp = pf.get("user")))
                obea.userId = MY_USER;
            else
                obea.userId = temp;

            if (null == (temp = pf.get("pin")))
                obea.pin = MY_PIN;
            else
                obea.pin = temp;

            if (null == (temp = pf.get("vendorId")))
                obea.vendorId = MY_VENDOR_ID;
            else
                obea.vendorId = temp;

            if (null == (temp = pf.get("vendorAppName")))
                obea.vendorAppName = MY_VENDOR_APP_NAME;
            else
                obea.vendorAppName = temp;

            if (null == (temp = pf.get("vendorAppVersion")))
                obea.vendorAppVersion = MY_VENDOR_APP_VERSION;
            else
                obea.vendorAppVersion = temp;

            /* TERMINAL DEVICE CONFIGURATION - START */
            TerminalDeviceType terminalDeviceType = MY_TERMINAL_DEVICE_TYPE;
            CatLevelIndicator terminalCatLevelIndicator = MY_TERMINAL_CAT_LEVEL_INDICATOR;

            if (null != (temp = pf.get("terminalDeviceType")))
                terminalDeviceType = (TerminalDeviceType)Enum.Parse(typeof(TerminalDeviceType), temp);

            if (null != (temp = pf.get("terminalCatLevelIndicator")))
                terminalCatLevelIndicator = (CatLevelIndicator)Enum.Parse(typeof(CatLevelIndicator), temp);

            obea.terminalDeviceConfiguration = new TerminalDeviceConfiguration(terminalDeviceType, terminalCatLevelIndicator);

            /* TERMINAL DEVICE CONFIGURATION - END */

            if (null == (temp = pf.get("terminalCardholderInputMode")))
                obea.terminalCardholderInputMode = MY_TERMINAL_CARDHOLDER_INPUT_MODE;
            else
                obea.terminalCardholderInputMode = (TerminalCardholderInputMode)Enum.Parse(typeof(TerminalCardholderInputMode), temp);

            if (null == (temp = pf.get("overrideLanguageLanguageCode")))
                obea.overrideDefaultTerminalLanguage.languageInformation.languageCode = MY_OVERRIDE_LANGUAGE_LANGUAGE_CODE;
            else
                obea.overrideDefaultTerminalLanguage.languageInformation.languageCode = temp;

            if (null == (temp = pf.get("overrideLanguageCountryCode")))
                obea.overrideDefaultTerminalLanguage.languageInformation.countryCode = MY_OVERRIDE_LANGUAGE_COUNTRY_CODE;
            else
                obea.overrideDefaultTerminalLanguage.languageInformation.countryCode = temp;

            String visaPref = pf.get("visaDebitNetworkPreference");
            String mastercardPref = pf.get("mastercardDebitNetworkPreference");
            String discoverPref = pf.get("discoverDebitNetworkPreference");
            if (visaPref != null || mastercardPref != null || discoverPref != null)
            {
                obea.overrideDebitNetworkPreferences = new OverrideDebitNetworkPreferences();
                obea.overrideDebitNetworkPreferences.visa = visaPref;
                obea.overrideDebitNetworkPreferences.mastercard = mastercardPref;
                obea.overrideDebitNetworkPreferences.discover = discoverPref;
            }

            if (null == (temp = pf.get("email")))
                obea.email = MY_EMAIL;
            else
                obea.email = temp;

            if (null == (temp = pf.get("serverType")))
                obea.paymentGatewayEnvironment = MY_SERVERTYPE;
            else
                obea.paymentGatewayEnvironment = temp;

            if ((null != (temp = pf.get("handleDigitalSignature"))) && ("true".Equals(temp)))
                obea.handleDigitalSignature = true;       /* We can handle digital signature */
            else
                obea.handleDigitalSignature = false;      /* We can't handle digital signature, generate blank */

            if (null != (temp = pf.get("signatureOption")))
                obea.signatureOption = temp;
            else
                obea.signatureOption = null;

            if ((null != (temp = pf.get("enableIvuExtension"))) && ("true".Equals(temp)))
                obea.enableIvuExtension = true;      
            else
                obea.enableIvuExtension = false;

            if (null != (temp = pf.get("healthCheckStaleTransactionTimeout")))
            {
                try
                {
                    obea.healthCheckStaleTransactionTimeout = Int32.Parse(temp);
                }
                catch (FormatException)
                {
                    Log(String.Format("Cannot interpret healthCheckStaleTransactionTimeout: {0}", temp));
                }
            }


            if (null != (temp = pf.get("healthCheckTaskTimeout")))
            {
                try
                {
                    obea.healthCheckTaskTimeout = Int32.Parse(temp);
                }
                catch (FormatException)
                {
                    Log(String.Format("Cannot interpret healthCheckTaskTimeout: {0}", temp));
                }
            }

            ProxyInfo proxyInfo = null;
            String proxyAddress = null;
            String proxyPort = null;
            String proxyUser = null;
            String proxyPassword = null;

            if (!(null == (temp = pf.get("proxyAddress"))))
                proxyAddress = temp;
            if (!(null == (temp = pf.get("proxyPort"))))
                proxyPort = temp;
            if (!(null == (temp = pf.get("proxyUser"))))
                proxyUser = temp;
            if (!(null == (temp = pf.get("proxyPassword"))))
                proxyPassword = temp;

            if (proxyAddress != null && proxyPort != null)
                proxyInfo = new ProxyInfo(proxyAddress, proxyPort, proxyUser, proxyPassword);

            obea.proxyInfo = proxyInfo;

            // adjust CWS log level if desired
            if (null != (temp = pf.get("logLevel")))
                obea.logLevel = temp;
            else
                obea.logLevel = null;

            OpenPaymentGatewayResults tr = m_CWS.OpenPaymentGateway(obea, null);
            MyOpenPaymentGatewayComplete(tr);
        }

        private void MyGetDeviceConfigResultsComplete(GetDeviceConfigResults deviceConfigResults)
        {
            Log("============ MyGetDeviceConfigResultsComplete ============ ");


            if (deviceConfigResults == null)
            {
                Log("Failed to get device configuration results");
                return;
            }

            if (deviceConfigResults.Error != null)
            {
                Log("Error: " + deviceConfigResults.Error);
            }

            Dispatcher.Invoke((Action)delegate
            {
                DeviceConfig dlg = new DeviceConfig(m_CWS, m_PaymentGatewayId, deviceConfigResults.connectionCriteria);
                dlg.Owner = this;
                dlg.ShowDialog();
                if (dlg.DoDeviceConfig())
                {
                    String[] providerTypes = dlg.getProviderTypes();
                    String[] connectionTypes = dlg.getConnectionTypes();
                    String[] deviceTypes = dlg.getDeviceTypes();

                    InetAddress inet = null;
                    String host = dlg.getHost();
                    int port = dlg.getPort();
                    if (host.Length > 0 && port >= 0)
                    {
                        inet = new InetAddress(host, port, InetAddress.EncryptionSchemefromString(dlg.getEncryption()));
                    }

                    SetDeviceConfig(providerTypes, connectionTypes, deviceTypes, inet);
                }
                else if (dlg.DoDeviceConfigReset())
                {
                    SetDeviceConfig(null);
                }
            });
        }

        private void MySetTerminalDeviceConfigurationComplete(SetTerminalDeviceConfigurationResults stdcr)
        {
            Log("============ MySetTerminalDeviceConfigurationComplete ============ ");
            Log(String.Format("Terminal Device Type: {0}", stdcr.terminalDeviceType));
            Log(String.Format("Terminal CAT Level Indicator: {0}", stdcr.terminalCatLevelIndicator));

            if (stdcr.Error != null)
            {
                Log("Error: " + stdcr.Error);
            }
        }

        private void MySetTerminalCardholderInputModeConfigurationComplete(SetTerminalCardholderInputModeConfigurationResults stcimcr)
        {
            Log("============ MySetTerminalCardholderInputModeConfigurationComplete ============ ");
            Log(String.Format("Terminal Cardholder Input Mode: {0}", stcimcr.terminalCardholderInputMode));

            if (stcimcr.Error != null)
            {
                Log("Error: " + stcimcr.Error);
            }
        }

        private void MyOverrideDefaultTerminalLanguageComplete(OverrideLanguageResults olr)
        {
            Log("============ MyOverrideDefaultTerminalLanguageComplete ============ ");
            if (olr.languageCode != null)
            {
                Log(String.Format("LanguageCode: {0}", olr.languageCode));
            }
            if (olr.countryCode != null)
            {
                Log(String.Format("CountryCode: {0}", olr.countryCode));
            }
            if (olr.Error != null)
            {
                Log("Error: " + olr.Error);
            }
        }

        private void MyOverrideDebitNetworkPreferencesComplete(OverrideDebitNetworkPreferencesResults odnpr)
        {
            Log("============ MyOverrideDebitNetworkPreferencesComplete ============ ");
            String error = odnpr.Error;
            if ((null != error) && (0 < error.Length))
                Log("Error: " + error);
            else
                Log("No error");
        }

        private void MyOverrideCvmOptionsComplete(OverrideCvmOptionsResults ocor)
        {
            Log("============ MyOverrideCvmOptionsComplete ============ ");
            String error = ocor.Error;
            if (error != null)
                Log("Error: " + error);
            else
                Log("No error");
        }

        private void MySetTransactionLaneNumberComplete(SetTransactionLaneNumberResults stlnr)
        {
            Log("============ MySetTransactionLaneNumberComplete ============ ");
            String error = stlnr.Error;
            if (error != null)
                Log("Error: " + error);
            else
                Log("No error");
        }

        private void MySetDeviceConfigComplete(SetDeviceConfigResults crc) 
        {
            Log("============ MySetDeviceConfigComplete ============ ");
            String statusDetails = crc.StatusDetails;
            if ((null != statusDetails) && (0 < statusDetails.Length))
                Log("Status Details: " + statusDetails);

            String error = crc.Error;
            if ((null != error) && (0 < error.Length))
                Log("Error: " + error);
            else
                Log("No error");
        }

        private void InitDeviceConnectionConfig() 
		{
            PropertyFile pf = LoadPropertyFile();           /* Load up the properties file */

            String providerTypesStr = pf.get("providerTypes");
            String[] providerTypes = String.IsNullOrWhiteSpace(providerTypesStr) ? null : providerTypesStr.Split(',');

            String connectionTypesStr = pf.get("connectionTypes");
            String[] connectionTypes = String.IsNullOrWhiteSpace(connectionTypesStr) ? null : connectionTypesStr.Split(',');

            String deviceTypesStr = pf.get("deviceTypes");
            String[] deviceTypes = String.IsNullOrWhiteSpace(deviceTypesStr) ? null : deviceTypesStr.Split(',');

            InetAddress inet = null;

            String host = pf.get("connectionHost");
            EncryptionScheme scheme = InetAddress.EncryptionSchemefromString(pf.get("encryptionScheme"));
            int port = -1;
            String portS = pf.get("connectionPort");
            try
            {
                port = int.Parse(portS);
            }
            catch (Exception)
            {
            }

            if (!String.IsNullOrWhiteSpace(host) && port > 0)
            {
                inet = new InetAddress(host, port, scheme);
            }

            //Thread.Sleep(1000);     /* Don't confuse the log */
            if (providerTypes != null || connectionTypes != null || deviceTypes != null || inet != null)
            {
                SetDeviceConfig(providerTypes, connectionTypes, deviceTypes, inet);
            }
		}

        private void SetDeviceConfig(String[] providerTypes, String[] connectionTypes, String[] deviceTypes, InetAddress inet)
        {
            DeviceConnectionCriteria connectionCriteria = new DeviceConnectionCriteria();
            connectionCriteria.providerTypes = providerTypes;
            connectionCriteria.connectionTypes = connectionTypes;
            connectionCriteria.deviceTypes = deviceTypes;
            connectionCriteria.inetAddress = inet;

            SetDeviceConfig(connectionCriteria);
        }

        private void SetDeviceConfig(DeviceConnectionCriteria criteria)
        {
            DeviceConfigArgs args = new DeviceConfigArgs();
            args.connectionCriteria = criteria;
            m_CWS.StartSetDeviceConfig(args, MyNotifyCWSEvent, MySetDeviceConfigComplete);
        }

        private void GetDeviceConfig()
        {
            m_CWS.StartGetDeviceConfig(MyNotifyCWSEvent, MyGetDeviceConfigResultsComplete);
        }

        private void StartSetTerminalDeviceConfiguration(TerminalDeviceType terminalDeviceType, CatLevelIndicator catLevelIndicator)
        {
            TerminalDeviceConfiguration terminalDeviceConfiguration = new TerminalDeviceConfiguration(terminalDeviceType, catLevelIndicator);
            m_CWS.StartSetTerminalDeviceConfiguration(terminalDeviceConfiguration, MyNotifyCWSEvent, MySetTerminalDeviceConfigurationComplete);
        }

        private void StartSetTerminalCardholderInputModeConfiguration(TerminalCardholderInputMode terminalCardholderInputMode)
        {
            TerminalCardholderInputModeConfiguration cardholderInputModeConfiguration = new TerminalCardholderInputModeConfiguration(terminalCardholderInputMode);
            m_CWS.StartSetTerminalCardholderInputMode(cardholderInputModeConfiguration, MyNotifyCWSEvent, MySetTerminalCardholderInputModeConfigurationComplete);
        }

        private void StartOverrideDefaultTerminalLanguage(String languageCode, String countryCode)
        {
            OverrideDefaultTerminalLanguage overrideTerminalLanguage = new OverrideDefaultTerminalLanguage();
            overrideTerminalLanguage.languageInformation = new LanguageInformation();
            overrideTerminalLanguage.languageInformation.languageCode = languageCode;
            overrideTerminalLanguage.languageInformation.countryCode = countryCode;
            m_CWS.StartOverrideDefaultTerminalLanguage(overrideTerminalLanguage, MyNotifyCWSEvent, MyOverrideDefaultTerminalLanguageComplete);
        }

        private void StartOverrideDebitNetworkPreferences(String visaPreference, String mastercardPreference, String discoverPreference)
        {
            OverrideDebitNetworkPreferences preferences = new OverrideDebitNetworkPreferences();
            if (visaPreference != null)
            {
                preferences.visa = visaPreference;
            }
            if (mastercardPreference != null)
            {
                preferences.mastercard = mastercardPreference;
            }
            if (discoverPreference != null)
            {
                preferences.discover = discoverPreference;
            }
            m_CWS.StartOverrideDebitNetworkPreferences(preferences, MyNotifyCWSEvent, MyOverrideDebitNetworkPreferencesComplete);
        }

        private void StartOverrideCvmOptions(String signatureOption)
        {
            OverrideCvmOptions overrideCvmOptions = new OverrideCvmOptions();
            overrideCvmOptions.signatureOption = signatureOption;
            m_CWS.StartOverrideCvmOptions(overrideCvmOptions, MyNotifyCWSEvent, MyOverrideCvmOptionsComplete);
        }

        private void StartSetTransactionLaneNumber(String laneNumber)
        {
            m_CWS.StartSetTransactionLaneNumber(laneNumber, MyNotifyCWSEvent, MySetTransactionLaneNumberComplete);
        }

        /*
         * OnClosing - windows/app is being closed.
         */
        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            m_CWS.Stop();
            m_CWS = null;
            Thread.Sleep(100);
            base.OnClosing(e);
        }

        private void InitQuickSelection()
        {
            m_QuickSelection = new QuickSelection();
            m_QuickSelection.selection1 = new Dictionary<String, String>();
            m_QuickSelection.selection1.Add("type", "PERCENTAGE");
            m_QuickSelection.selection1.Add("currencyCode", "USD");
            m_QuickSelection.selection1.Add("value", "13");
            m_QuickSelection.selection2 = new Dictionary<String, String>();
            m_QuickSelection.selection2.Add("type", "PERCENTAGE");
            m_QuickSelection.selection2.Add("currencyCode", "USD");
            m_QuickSelection.selection2.Add("value", "18");
            m_QuickSelection.selection3 = new Dictionary<String, String>();
            m_QuickSelection.selection3.Add("type", "PERCENTAGE");
            m_QuickSelection.selection3.Add("currencyCode", "USD");
            m_QuickSelection.selection3.Add("value", "23");
        }

        /*
         * InitPrintPaymentArgs - these are passed to PrintReceipt
         */
        private void InitPrintPaymentArgs()
        {
            m_PrintPaymentArgs = new PrintPaymentArgs();

            m_PrintPaymentArgs.paymentGatewayId = m_PaymentGatewayId;

            /* AccountInfo - if we don't initialize AccountInfo */
            /* The name, adddress etc that is printed on the receipt */
            /* will come from Converge. */
            /* This is how you can override those values. */
            m_PrintPaymentArgs.accountInfo = m_AccountInfo;

            m_PrintPaymentArgs.currencyFormatPattern = "###,###.## USD";    /* Add currency symbol to pattern */


            /* Currenly not used, but future versions will use these terms */
            /* when printing a receipt. */

            Dictionary<String, String> terms = new Dictionary<String, String>();
            terms.Add("CARD", "CreditCard");
            terms.Add("START", "Start");
            terms.Add("EXPIRY", "Expire");
            terms.Add("SALE", "Sale");
            terms.Add("REFUND", "Refundent");
            terms.Add("CASH", "Cashless");
            terms.Add("CHECK", "Checkered");
            terms.Add("GRATUITY", "Tipsy");
            terms.Add("TOTAL", "Totality");
            terms.Add("SUBTOTAL", "not the whole amount");
            terms.Add("DISCOUNT", "Discounted");
            terms.Add("TAX", "Taxi");
            m_PrintPaymentArgs.receiptTerms = terms;
        }

        private void FillTranType()
        {
            Dispatcher.Invoke((Action)delegate
            {
                cbox_TranType.Items.Clear();
                cbox_TranType.Items.Add(TransactionType.SALE);
                cbox_TranType.Items.Add(TransactionType.STANDALONE_REFUND);
                cbox_TranType.Items.Add(TransactionType.LINKED_REFUND);
                cbox_TranType.Items.Add(TransactionType.VOID);
                cbox_TranType.Items.Add(TransactionType.PRE_AUTH);
                cbox_TranType.Items.Add(TransactionType.PRE_AUTH_INCREMENT);
                cbox_TranType.Items.Add(TransactionType.PRE_AUTH_COMPLETE);
                cbox_TranType.Items.Add(TransactionType.PRE_AUTH_DELETE);
                cbox_TranType.Items.Add(TransactionType.MONEY_TRANSFER_OCT);
                cbox_TranType.Items.Add(TransactionType.TIP_ADJUST);
                cbox_TranType.Items.Add(TransactionType.BALANCE_INQUIRY);
                cbox_TranType.Items.Add(TransactionType.OCT_FUNDING);
                cbox_TranType.SelectedIndex = 0;
            });
        }

        private void InitRecentTransactionOutcomes()
        {
            // This empty outcome is always the first entry in the recent transaction combobox.
            m_RecentTransactionOutcomes = new List<PaymentTransactionData>();
            m_RecentTransactionOutcomes.Add(new PaymentTransactionData());
        }

        private void AddRecentTransactionOutcome(PaymentTransactionData outcome)
        {
            // Add at 1 instead of 0 to preserve the empty outcome that is always first in the list
            m_RecentTransactionOutcomes.Insert(1, outcome);

            // keep the recent txn list to a manageable number
            int count = m_RecentTransactionOutcomes.Count;
            if (count > 11)
            {
                m_RecentTransactionOutcomes.RemoveAt(count - 1);
            }
        }

        private void FillRecentTransactionOutcomes()
        {
            Dispatcher.Invoke((Action)delegate
            {
                cbox_RecentTransactions.Items.Clear();

                foreach (PaymentTransactionData data in m_RecentTransactionOutcomes)
                {
                    StringBuilder sb = new StringBuilder();
                    if (data.transactionType != null)
                    {
                        sb.Append(data.transactionType);
                        sb.Append(" ");
                    }
                    if (data.id != null)
                    {
                        sb.Append(data.id);
                    }

                    cbox_RecentTransactions.Items.Add(sb.ToString());
                }

                cbox_RecentTransactions.SelectedIndex = 0;
            });
        }

        private void cbox_RecentTransactions_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int index = cbox_RecentTransactions.SelectedIndex;
            if ((index >= 0) && (index < cbox_RecentTransactions.Items.Count))
            {
                PaymentTransactionData data = m_RecentTransactionOutcomes[index];
                tb_OriginalTxnId.Text = data.id;
                tb_TokenizedCard.Text = data.tokenizedCard;
                tb_TokenExpirationDate.Text = data.tokenExpirationDate;
                tb_OarData.Text = data.oarData;
                tb_Ps2000Data.Text = data.ps2000Data;
            }
        }

        private Boolean isPreviousTransactionTipAdjustable()
        {
            return (null != m_OriginalTransId) && (0 < m_OriginalTransId.Length) &&
                (null != m_TransData) && (TransactionType.SALE.Equals(m_TransData.transactionType)) && (("yes").Equals(m_TransData.approved));
        }

        private void FillTenderType()
        {
            cbox_TenderType.Items.Add(TenderType.CARD);
            cbox_TenderType.Items.Add(TenderType.CASH);
            cbox_TenderType.SelectedIndex = 0;
        }

        private void FillEntryType()
        {
            cb_EntryType.Items.Add(CardEntryType.Default);
            cb_EntryType.Items.Add(CardEntryType.TOKEN);
            cb_EntryType.Items.Add(CardEntryType.MANUALLY_ENTERED);
            cb_EntryType.SelectedIndex = 0;
        }

        private void FillCurrency()
        {
            foreach (CurrencyCode cc in Enum.GetValues(typeof(CurrencyCode)))
            {
                cb_Currency.Items.Add(cc.ToString());
            }
            cb_Currency.SelectedIndex = 0;
        }

        private void FillCardType()
        {
            cbox_CardType.Items.Add(CardType.DEFAULT);
            cbox_CardType.Items.Add(CardType.CREDIT);
            cbox_CardType.Items.Add(CardType.DEBIT);
            cbox_CardType.SelectedIndex = 0;
        }

        private void FillSignatureOption()
        {
            signatureOptionCheckBox.SetCurrentValue(CheckBox.IsCheckedProperty, false);
            signatureOptionCheckBox.Checked += SignatureOptionCheckBox_Checked;
            signatureOptionCheckBox.Unchecked += SignatureOptionCheckBox_Checked;

            signatureOptionComboBox.Items.Add(SignatureOption.DISABLED);
            signatureOptionComboBox.Items.Add(SignatureOption.OPTIONAL);
            signatureOptionComboBox.Items.Add(SignatureOption.ENABLED_MSR_ONLY);
            signatureOptionComboBox.Items.Add(SignatureOption.ENABLED_ALL);
            signatureOptionComboBox.SelectedIndex = 0;
            signatureOptionComboBox.IsEnabled = (signatureOptionCheckBox.IsChecked ?? false) ? true : false;
        }

        private void SignatureOptionCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            signatureOptionComboBox.IsEnabled = (signatureOptionCheckBox.IsChecked ?? false) ? true : false;
        }

        private void Log(String msg)
        {
            Dispatcher.Invoke((Action)delegate
            {
                lb_Log.Items.Add(String.Format("{0} - {1}", m_logcnt++, msg));
                int cnt = lb_Log.Items.Count;
                lb_Log.SelectedIndex = cnt - 1;
                lb_Log.ScrollIntoView(lb_Log.Items[lb_Log.Items.Count - 1]);
            });
        }

        private void Log(String desc, Money money)
        {
            if (null != money)
            {
                StringBuilder sb = new StringBuilder();
                sb.Append(desc)
                  .Append(" ")
                  .Append(money.value)
                  .Append(" ")
                  .Append(money.currencyCode);

                Log(sb.ToString());
            }
        }
        private void Log(String desc, String value)
        {
            if (null != value && (0 < value.Length))
            {
                StringBuilder sb = new StringBuilder();
                sb.Append(desc)
                  .Append(" ")
                  .Append(value);

                Log(sb.ToString());
            }
        }

        private void SetUIChanId(String chanId)
        {
            if (!string.IsNullOrEmpty(chanId))
            {
                m_ChanId = chanId;
            }
        }

        /*
         * SetToken - When we generate a token, we don't save it anywhere but in the UI. 
         * A real app would store it somewhere permanant, like a database. 
         * For our purposes the UI is good enough.  Once you generate a token you can 
         * perform a tokenized sale, the code below with get the token from the UI's text box 
         */
        private void SetToken(string token)
        {
            Dispatcher.Invoke((Action)delegate
            {
                tb_TokenizedCard.Text = token;                     /* Update token in UI */
            });
        }

        private void SetTokenExpirationDate(string tokenExpDate)
        {
            Dispatcher.Invoke((Action)delegate
            {
                tb_TokenExpirationDate.Text = tokenExpDate;        /* Update token exp date in UI */
            });
        }

        private void SetOarData(string oarData)
        {
            Dispatcher.Invoke((Action)delegate
            {
                tb_OarData.Text = oarData;                         /* Update oar data in UI */
            });
        }

        private void SetPs2000Data(string ps2000Data)
        {
            Dispatcher.Invoke((Action)delegate
            {
                tb_Ps2000Data.Text = ps2000Data;                   /* Update ps2000 data in UI */
            });
        }

        private void SetTip(string tip)
        {
            Dispatcher.Invoke((Action)delegate
            {
                tb_Tip.Text = tip; 
            });
        }

        private void b_CalculateExtraCharge_Click(object sender, RoutedEventArgs e)
        {
            if (null == m_PaymentGatewayId)
            {
                MessageBox.Show("No backend id", "ERROR");
                return;
            }
            String myAmount = tb_Amount.Text;
            String myCurrency = cb_Currency.SelectedItem.ToString();

            m_OriginalTransId = tb_OriginalTxnId.Text;

            Log(String.Format("============ Calculate Extra Charge Request: amount {0} currency {1} ============ ", myAmount, myCurrency));

            CalculateExtraChargeArgs bea = new CalculateExtraChargeArgs();
            bea.baseTransactionAmount = new Money(Money.CurrencyCodefromString(myCurrency), myAmount);

            bea.paymentGatewayId = m_PaymentGatewayId;
            SetTransState(TransState.InProgress);

            bool? check = cb_Async.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                m_IsAsync = true;
                m_CWS.StartExtraChargeCalculation(bea, MyNotifyCWSEvent, CalculateExtraChargeCompleteCallback);
            }
            else
            {
                m_IsAsync = false;
                new Thread(() =>
                {
                    /* We expect lots of callbacks.  We can't wait on the GUI thread */
                    /* because our callback calls Log and Log will hang */
                    /* Fire up a separate thread */

                    PaymentTransactionResults tr = m_CWS.CalculateExtraCharge(bea, MyNotifyCWSEvent);
                    MyPaymentComplete(tr);
                }).Start();
            }
        }

        /*
         * They clicked the "Start Transaction" button.
         * Figure out what options, tax, tip etc, then call StartPaymentTransaction
         */
        private void b_Transaction_Click(object sender, RoutedEventArgs e)
        {
            if (null == m_PaymentGatewayId)
            {
                MessageBox.Show("No backend id", "ERROR");
                return;
            }
            String myAmount = tb_Amount.Text;
            String myCurrency = cb_Currency.SelectedItem.ToString();
            String myTranType = cbox_TranType.SelectedItem.ToString();
            String myTender = cbox_TenderType.SelectedItem.ToString();

            m_OriginalTransId = tb_OriginalTxnId.Text;

            Log(String.Format("============ Transaction Request: amount {0} currency {1} Trantype {2} Tender {3} ============ ", myAmount, myCurrency, myTranType, myTender));

            PaymentArgs bea = new PaymentArgs();
            Money m = new Money(Money.CurrencyCodefromString(myCurrency), myAmount);
            bea.baseTransactionAmount = m;
            bea.tenderedAmount = m;

            bea.tenderType = myTender;
            bea.transactionType = myTranType;

            bea.paymentGatewayId = m_PaymentGatewayId;
            bea.originalTransId = m_OriginalTransId;

            /* See if we should include tax, tip and discount now */

            bool? check;
            check = cb_Tax.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                Money t = new Money(Money.CurrencyCodefromString(myCurrency), tb_Tax.Text);
                bea.taxAmounts = new Money[1];
                bea.taxAmounts[0] = t;
            }

            check = cb_IsTaxInclusive.IsChecked;
            if ((check.HasValue) && (check.Value))
                bea.isTaxInclusive = true;
            else
                bea.isTaxInclusive = false;

            check = cb_VerifyTipAdjust.IsChecked;
            if ((check.HasValue) && (check.Value))
                bea.verifyTipAdjustByIssuer = true;
            else
                bea.verifyTipAdjustByIssuer = false;

            check = cb_Tip.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                Money t = new Money(Money.CurrencyCodefromString(myCurrency), tb_Tip.Text);
                bea.gratuityAmount = t;
            }

            check = cb_Discount.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                Money d = new Money(Money.CurrencyCodefromString(myCurrency), tb_Discount.Text);
                bea.discountAmounts = new Money[1];
                bea.discountAmounts[0] = d;
            }

            check = cb_Note.IsChecked;
            if ((check.HasValue) && (check.Value))
                bea.note = tb_Note.Text;

            check = cb_MTR.IsChecked;
            if ((check.HasValue) && (check.Value))
                bea.merchantTransactionReference = tb_MTR.Text;

            check = cb_InvoiceNum.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                bea.invoiceNumber = tb_InvoiceNum.Text;
            }

            check = cb_Force.IsChecked;
            if ((check.HasValue) && (check.Value))
                bea.forceTransaction = true;
            else
                bea.forceTransaction = false;

            check = cb_GenerateToken.IsChecked;
            if ((check.HasValue) && (check.Value))
                bea.generateToken = true;
            else
                bea.generateToken = false;

            check = cb_AddToken.IsChecked;
            if ((check.HasValue) && (check.Value))
                bea.addToken = true;
            else
                bea.addToken = false;

            check = cb_TipOnReader.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                bea.gratuityFlowEnabled = true;
                check = cb_CustomAmount.IsChecked;
                if ((check.HasValue) && (check.Value))
                    bea.gratuityCustomAmountEntryAllowed = true;
                else
                    bea.gratuityCustomAmountEntryAllowed = false;

                check = cb_TipDefaults.IsChecked;
                if ((check.HasValue) && (check.Value))  /* Use the default amounts? */
                    bea.gratuityQuickSelections = null;  /* Yes */
                else
                {
                    QuickValues dlg = new QuickValues(myCurrency);
                    dlg.Owner = this;
                    dlg.ShowDialog();
                    QuickSelection qs = dlg.getQuickSelection();
                    if (null != qs)
                        bea.gratuityQuickSelections = qs;
                    else
                        bea.gratuityQuickSelections = null; 
                }
            }
            else
            {
                bea.gratuityFlowEnabled = false;
            }

            check = cb_PartialApproval.IsChecked;
            if ((check.HasValue) && (check.Value))
                bea.partialApprovalAllowed = true;
            else
                bea.partialApprovalAllowed = false;

            check = cb_SkipAmountOnCardEntryPrompt.IsChecked;
            bea.skipAmountOnCardEntryPrompt = check.HasValue && check.Value;

            String entryType = cb_EntryType.SelectedItem.ToString();
            if (CardEntryType.TOKEN.Equals(entryType))             /* Tokenized? */
            {
                bea.tokenizedCardNumber = tb_TokenizedCard.Text;    /* Yes, get token from UI */
                bea.tokenExpirationDate = tb_TokenExpirationDate.Text;
                bea.oarData = tb_OarData.Text;
                bea.ps2000Data = tb_Ps2000Data.Text;
                bea.cardEntryTypes = new string[1];
                bea.cardEntryTypes[0] = CardEntryType.TOKEN;
            }
            else if (!CardEntryType.Default.Equals(entryType))
            {
                bea.cardEntryTypes = new string[1];
                bea.cardEntryTypes[0] = entryType;
            }

            String cardType = cbox_CardType.SelectedItem.ToString();
            if (!CardType.DEFAULT.Equals(cardType))
                bea.cardType = cardType;

            bea.avsFields = GetAVSFields();

            check = signatureOptionCheckBox.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                string signatureOption = signatureOptionComboBox.SelectedItem.ToString();
                bea.signatureOption = signatureOption;
            }

            check = cb_OctRefNumber.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                bea.octRefNumber = tb_OctRefNumber.Text;
            }
            
            SetAdditionalTransactionPropertiesIfProvided(bea);

            SetTransState(TransState.InProgress);
            check = cb_Async.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                m_IsAsync = true;

                if (TransactionType.TIP_ADJUST.Equals(myTranType))
                {
                    m_CWS.StartTipAdjust(m_PaymentGatewayId, m_OriginalTransId, bea.gratuityAmount, bea.verifyTipAdjustByIssuer,
                        MyNotifyCWSEvent, MyPaymentComplete);
                }
                else
                {
                    m_CWS.StartPaymentTransaction(bea, MyNotifyCWSEvent, MyPaymentComplete);
                }
            }
            else
            {
                m_IsAsync = false;
                new Thread(() =>
                {
                    /* We expect lots of callbacks.  We can't wait on the GUI thread */
                    /* because our callback calls Log and Log will hang */
                    /* Fire up a separate thread */

                    PaymentTransactionResults tr;
                    if (TransactionType.TIP_ADJUST.Equals(myTranType))
                    {
                        tr = m_CWS.TipAdjust(m_PaymentGatewayId, m_OriginalTransId, bea.gratuityAmount, bea.verifyTipAdjustByIssuer, MyNotifyCWSEvent);
                    }
                    else
                    {
                        tr = m_CWS.PaymentTransaction(bea, MyNotifyCWSEvent);
                    }
                    MyPaymentComplete(tr);
                }).Start();
            }
        }

        // sets properties common to all transaction types
        private void SetAdditionalTransactionPropertiesIfProvided(PaymentArgs bea)
        {
            bool? check = cb_LaneNumber.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                bea.laneNumber = tb_LaneNumber.Text;
            }
        }

        private String generateOctRefNumber()
        {
            // ssl_oct_ref_number is a required unique reference number and should be alphanumeric, max length = 16 characters.
            Guid guid = Guid.NewGuid();
            int guidHashCode = guid.GetHashCode();
            String octRefNumber = guidHashCode.ToString();
            octRefNumber = octRefNumber.Replace("-", "");
            int paddingLength = 16 - octRefNumber.Length;
            if (paddingLength > 0)
            {
                String padding = new String('0', paddingLength);
                octRefNumber = padding + octRefNumber;
            }
            Log("generateOctRefNumber: " + octRefNumber);
            return octRefNumber;
        }

        private void b_GenerateOctRefNumber_Click(object sender, RoutedEventArgs e)
        {
            Log("------------- Generate OCT Ref # Clicked ------------");
            tb_OctRefNumber.Text = generateOctRefNumber();
        }

        private void b_Cancel_Click(object sender, RoutedEventArgs e)
        {
            Log("------------- Cancel Transaction Clicked ------------");
            Log("The current id is " + m_ChanId);
            string chanId = m_ChanId;
            if (null != m_PaymentGatewayId)
            {
                SetTransState(TransState.InProgress);
                bool? check = cb_Async.IsChecked;
                if ((check.HasValue) && (check.Value))
                {
                    m_CWS.StartCancelPaymentTransaction(m_PaymentGatewayId, chanId, MyNotifyCWSEvent, MyPaymentComplete);
                }
                else
                {
                    PaymentTransactionResults tr = m_CWS.CancelPaymentTransaction(m_PaymentGatewayId, chanId, null);
                    MyPaymentComplete(tr);
                }
            }
            else
            {
                MessageBox.Show("Null PaymentGatewayId", "Error");
            }
        }

        private AvsFields GetAVSFields()
        {
            AvsFields info = new AvsFields();
            bool set = false;

            bool? check;

            check = cb_Address.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                info.cardHolderAddress = tb_Address.Text;
                set = true;
            }

            check = cb_Zip.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                info.cardHolderZip = tb_Zip.Text;
                set = true;
            }

            check = cb_FirstName.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                info.cardHolderFirstName = tb_FirstName.Text;
                set = true;
            }

            check = cb_LastName.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                info.cardHolderLastName = tb_LastName.Text;
                set = true;
            }

            return set ? info : null ;
        }

        /*
         * They clicked Continue Transaction
         * This implied we received a response with "Completed:false" and "requiredInformation".
         * A real app would respond to that response.  This test app assumes the user has observed
         * what "requiredInformation" was returned, and will select the correct response.
         */
        private void b_ContinueCardData_Click(object sender, RoutedEventArgs e)
        {
            Log("------------- Continue Transaction Clicked ------------");
            string chanId = m_ChanId;

            {
                /* add any required parameters - using the checkboxes */

                Dictionary<string, string> info = new Dictionary<string, string>();

                bool? check;

                /* NOTE:  This app passes in Tax, Tip and Discount during */
                /* StartPaymentTransaction, so this app should never receive */
                /* "requiredInformation=tax, tip or discount */

                check = cb_Tax.IsChecked;
                if ((check.HasValue) && (check.Value))
                    info["Tax"] = tb_Tax.Text;

                check = cb_Tip.IsChecked;
                if ((check.HasValue) && (check.Value))
                    info["Tip"] = tb_Tip.Text;

                check = cb_Discount.IsChecked;
                if ((check.HasValue) && (check.Value))
                    info["Discount"] = tb_Discount.Text;

                check = cb_Voice.IsChecked;
                if ((check.HasValue) && (check.Value))
                    info["VoiceReferral"] = tb_Voice.Text;

                bool? signatureCheck = cb_Signature.IsChecked;
                bool? cancelSignatureCheck = cb_SignatureCancel.IsChecked;
                if ((signatureCheck.HasValue) && (signatureCheck.Value))
                {
                    string fullpath = GetFullPath(tb_Signature.Text);
                    string sigbytes = GetBase64Bytes(fullpath);
                    if (null != sigbytes)
                        info["DigitalSignature"] = sigbytes;
                    else
                        info["DigitalSignature"] = "";      /* Force a card tender cancelSignature */
                }
                else if ((cancelSignatureCheck.HasValue) && (cancelSignatureCheck.Value))
                {
                    info["DigitalSignature"] = "";      /* Force a card tender cancelSignature */
                }

                check = cb_CardPresent.IsChecked;
                if ((check.HasValue) && (check.Value))
                {
                    check = cb_CPValue.IsChecked;
                    if ((check.HasValue) && (check.Value))
                        info["CardPresent"] = "true";
                    else
                        info["CardPresent"] = "false";
                }
                check = cb_SigVerified.IsChecked;
                if ((check.HasValue) && (check.Value))
                {
                    check = cb_SVValue.IsChecked;
                    if ((check.HasValue) && (check.Value))
                        info["SignatureVerification"] = "true";
                    else
                        info["SignatureVerification"] = "false";
                }

                check = cb_aidSelected.IsChecked;
                if ((check.HasValue) && (check.Value))
                {
                    if (cb_aidSelection.SelectedItem != null)
                    {
                        String selectedAppId = ((EmvApplication)cb_aidSelection.SelectedItem).Aid;
                        info["EmvApplicationIdSelected"] = selectedAppId != null && selectedAppId.Length > 0 ? selectedAppId : null;
                    }
                }

                check = cb_Zip.IsChecked;
                if ((check.HasValue) && (check.Value))
                { 
                    info["RequireAvsZipCode"] = tb_Zip.Text;
                }

                check = cb_paymentTypeSelected.IsChecked;
                if ((check.HasValue) && (check.Value))
                {
                    if (cb_paymentTypeSelection.SelectedItem != null)
                    {
                        String paymentTypeSelectedString = ((PaymentType) cb_paymentTypeSelection.SelectedItem).ToString();
                        info["PaymentTypeSelected"] = paymentTypeSelectedString != null && paymentTypeSelectedString.Length > 0 ? paymentTypeSelectedString : null;
                    }
                }

                check = cb_accountTypeSelected.IsChecked;
                if ((check.HasValue) && (check.Value))
                {
                    if (cb_accountTypeSelection.SelectedItem != null)
                    {
                        String accountTypeSelectedString = ((AccountType)cb_accountTypeSelection.SelectedItem).ToString();
                        info["AccountTypeSelected"] = accountTypeSelectedString != null && accountTypeSelectedString.Length > 0 ? accountTypeSelectedString : null;
                    }
                }

                SetTransState(TransState.InProgress);
                check = cb_Async.IsChecked;
                if ((check.HasValue) && (check.Value))
                {
                    m_CWS.StartContinuePaymentTransaction(m_PaymentGatewayId, chanId, info, MyNotifyCWSEvent, MyPaymentComplete);
                }
                else
                {
                    new Thread(() =>
                    {
                        PaymentTransactionResults tr = m_CWS.ContinuePaymentTransaction(m_PaymentGatewayId, chanId, info, MyNotifyCWSEvent);
                        MyPaymentComplete(tr);
                    }).Start();
                }
            }
        }

        /*
         * GetBase64Bytes - base 64 encode a file
         */
        private string GetBase64Bytes(string path)
        {
            try
            {
                byte[] bytes = File.ReadAllBytes(path);
                String rv = Convert.ToBase64String(bytes);
                return rv;  
            }
            catch (Exception )
            {
                Log("GetBase64 exception");
            }
            return null;
        }

        public void MyOpenPaymentGatewayComplete(OpenPaymentGatewayResults ober)
        {
            Log("----------- OpenPaymentGateway Complete ------------");
            OpenPaymentData obed = ober.OpenPaymentData;
            if (null != obed)
            {
                m_PaymentGatewayId = obed.paymentGatewayId;
                Log(String.Format("paymentGatewayId {0}", m_PaymentGatewayId));
                Log(String.Format("result {0}", obed.result));
            }
            else
            {
                Log("No OpenPaymentGatewayData");
            }

            AccountInfo accountInfo = ober.AccountInfo;
            if (null != accountInfo)
            {
                updateAccountInformation(accountInfo);
            }

            ShowResults(ober);
        }

        private void updateAccountInformation(AccountInfo acctInfo)
        {
            m_AccountInfo = acctInfo;
            Log("----- AccountInfo -----");
            Log(String.Format("Name:  {0}", acctInfo.name));
            Log(String.Format("Address:  {0}", acctInfo.address1));
            Log(String.Format("City:  {0}", acctInfo.city));
            Log(String.Format("State:  {0}", acctInfo.stateProvince));
            Log(String.Format("Zip:  {0}", acctInfo.postalCode));
            Log(String.Format("Contact Phone:  {0}", acctInfo.contactPhone));
            Log(String.Format("BusinessEmail:  {0}", acctInfo.businessEmail));
            Log(String.Format("MarketSegment:  {0}", acctInfo.marketSegment));
            Log(String.Format("GratuitySupport:  {0}", acctInfo.isGratuitySupported));
            Log(String.Format("CvvEnabled:  {0}", acctInfo.isCvvEnabled));
            Log(String.Format("AvsEnabled:  {0}", acctInfo.isAvsEnabled));
            Log(String.Format("OctDisbursementEnabled:  {0}", acctInfo.isOctDisbursementEnabled));
            Log(String.Format("OctFundingEnabled:  {0}", acctInfo.isOctFundingEnabled));
            Log(String.Format("HealthcareEnabled:  {0}", acctInfo.isHealthcareEnabled));
            Log(String.Format("PinlessDebitEnabled:  {0}", acctInfo.isPinlessDebitEnabled));
            if (acctInfo.isPinlessDebitEnabled)
            {
                Log(String.Format("PinlessDebitThresholdCurrency:  {0}", acctInfo.pinlessDebitThresholdCurrency));
                Log(String.Format("PinlessDebitThresholdAmount:  {0}", acctInfo.pinlessDebitThresholdAmount));
            }

            Log(String.Format("Credit Surcharge Allowed:  {0}", acctInfo.isCreditSurchargeAllowed));
            if (acctInfo.isCreditSurchargeAllowed)
            {
                Log(String.Format("Credit Surcharge Percent:  {0}", acctInfo.creditSurchargePercent));
            }


            Log(String.Format("Service Fee Allowed:  {0}", acctInfo.isServiceFeeAllowed));
            if (acctInfo.isServiceFeeAllowed)
            {
                Log(String.Format("Service Fee Type:  {0}", acctInfo.serviceFeeType));
                switch (acctInfo.serviceFeeType)
                {
                    case ServiceFeeType.FIXED_AMOUNT:
                        Log(String.Format("Service Fee Amount:  {0}", acctInfo.serviceFeeAmount));
                        break;
                    case ServiceFeeType.PERCENTAGE:
                        Log(String.Format("Service Fee Percent:  {0}", acctInfo.serviceFeePercent));
                        break;
                }
                Log(String.Format("Service Fee Refund Allowed:  {0}", acctInfo.serviceFeeAllowRefund));
            }

            Log(String.Format("Converge Terminal Signature Option: {0}", acctInfo.signatureOptionTerminal));
            Log(String.Format("Applied Signature Option: {0}", acctInfo.signatureOption));

            if (!m_AccountInfo.isCvvEnabled)
            {
                cb_EntryType.Items.Remove(CardEntryType.MANUALLY_ENTERED);
            }

            /*
            if (!m_AccountInfo.isAvsEnabled)
            {
                cb_Address.IsEnabled = false;
                tb_Address.IsEnabled = false;
                cb_Zip.IsEnabled = false;
                tb_Zip.IsEnabled = false;
            }
            */
        }

        public void MyPrintReceiptComplete(PrintReceiptResults scr)
        {
            Log("----------- PrintReceipt Complete ------------");
            ShowResults(scr);
        }

        private bool CanUndoTransaction(string tt)
        {
            if (TransactionType.VOID.Equals(tt))
                return false;
            if (TransactionType.PRE_AUTH_DELETE.Equals(tt))
                return false;
            if (TransactionType.PRE_AUTH_COMPLETE.Equals(tt))
                return false;
            if (TransactionType.LINKED_REFUND.Equals(tt))
                return false;
            if (TransactionType.TIP_ADJUST.Equals(tt))
                return false;
            if (TransactionType.OCT_FUNDING.Equals(tt))
                return false;

            return true;
        }

        /*
         * MyPaymentComplete - called upon completion of payment transaction.
         * Extract the important bits and show them.
         */
        public void MyPaymentComplete(PaymentTransactionResults paymentResults)
        {
            Log("----------- Payment Complete ------------");
            String chanId = paymentResults.GetChannelId();
            SetUIChanId(chanId);
            PaymentTransactionData paymentData = paymentResults.PaymentTransactionData;
            if (null != paymentData)
            {
                m_TransData = paymentData;
                Log("approved", paymentData.approved);

                if (null != paymentData.id)
                {
                    Log("id", paymentData.id);
                    m_OriginalTransId = paymentData.id;
                    AddRecentTransactionOutcome(paymentData);
                    FillRecentTransactionOutcomes();
                }

                Log(String.Format("result {0}", paymentData.result));
                Log(String.Format("Approval Code: {0}", paymentData.authCode));
                if (paymentData.result.Equals("TIMEOUT"))
                    m_CWS.StartCancelPaymentTransaction(m_PaymentGatewayId, m_ChanId, MyNotifyCWSEvent, MyPaymentComplete);

                Log("tokenizedCard", paymentData.tokenizedCard);
                Log("tokenExpirationDate", paymentData.tokenExpirationDate);
                Log("merchant transaction reference", paymentData.merchantTransactionReference);
                Log("note", paymentData.note);
                Log("oarData", paymentData.oarData);
                Log("ps2000Data", paymentData.ps2000Data);
                Log("balanceDue", paymentData.balanceDue);
                Log("amountAuthorized", paymentData.amountAuthorized);

                Log("creditSurchargeStatus", paymentData.creditSurchargeStatus);
                Log("creditSurcharge", paymentData.creditSurcharge);
                Log("avsResponse", paymentData.avsResponse);

                if (!string.IsNullOrEmpty(paymentData.serviceFeeTransactionId))
                {
                    Log("serviceFeeTransactionId", paymentData.serviceFeeTransactionId);
                    Log("serviceFeeTransactionApprovalCode", paymentData.serviceFeeTransactionApprovalCode);
                    Log("serviceFeeTransactionDate", paymentData.serviceFeeTransactionDate);
                    Log("serviceFeeAmount", paymentData.serviceFeeAmount);
                }

                if ((null != paymentData.tokenizedCard) && (0 < paymentData.tokenizedCard.Length))
                {
                    SetToken(paymentData.tokenizedCard);

                    if ((null != paymentData.oarData) && (0 < paymentData.oarData.Length))
                    {
                        SetOarData(paymentData.oarData);
                    }
                    if ((null != paymentData.ps2000Data) && (0 < paymentData.ps2000Data.Length))
                    {
                        SetPs2000Data(paymentData.ps2000Data);
                    }
                }
                if ((null != paymentData.tokenExpirationDate) && (0 < paymentData.tokenExpirationDate.Length))
                    SetTokenExpirationDate(paymentData.tokenExpirationDate);

                Log("date", paymentData.date);
                if (null != paymentData.errors)
                {
                    for (int i = 0; i < paymentData.errors.Length; i++)
                    {
                        Log(String.Format("Error[{0}] =  {1}", i, paymentData.errors[i]));
                    }
                }

                Log("maskedPan", paymentData.maskedPan);
                Log("firstName", paymentData.firstName);
                Log("lastName", paymentData.lastName);
                Log("tenderType", paymentData.tenderType);
                Log("isFallback", paymentData.isFallback.ToString());

                signatureBitmap sb = paymentData.signatureBitmap;
                if (null != sb)
                {
                    Log("Signature format", sb.format);
                    Log("Signature data", sb.data);
                }

                Money ga = paymentData.gratuityAmount;
                if (null != ga)
                {
                    Log("Gratuity", ga);
                    SetTip(ga.value.ToString());
                }

                Log("octRefNumber", paymentData.octRefNumber);
                Log("tipAmount", paymentData.tipAmount);
                Log("baseAmount", paymentData.baseAmount);

                Log("octFundingTxnCount", paymentData.octFundingTxnCount);
                Log("octFundingTxnAmount", paymentData.octFundingTxnAmount);
                Log("octFundingGiftCardCount", paymentData.octFundingGiftCardCount);
                Log("octFundingGiftCardAmount", paymentData.octFundingGiftCardAmount);
               
                Log("referenceNumber", paymentData.referenceNumber);
                Log("debitAccountType", paymentData.debitAccountType);
                if (null != paymentData.accountBalance)
                {
                    Log(String.Format("accountBalance {0}", paymentData.accountBalance.getAmountInMinorUnits()));
                }

                Log("AID:", paymentData.iccAid);
                Log("TVR:", paymentData.iccTvr);
                Log("TSI:", paymentData.iccTsi);

                Log("AC:", paymentData.iccAc);
                Log("ARC:", paymentData.iccArc);
                Log("ATC:", paymentData.iccAtc);
            
                Log("CSN:", paymentData.iccCsn);

                Log("IAD:", paymentData.iccIad);
            }

            AccountInfo acctInfo = paymentResults.AccountInfo;
            if (null != acctInfo)
            {
                updateAccountInformation(acctInfo);
            }
            /*
             * Does CWS require additional information?
             * Just show it in the UI
             */
            String[] req = paymentResults.GetRequiredInformation();
            if (null != req)
            {
                Log("---- Information Required ----");
                for (int i = 0; i < req.Length; i++)
                {
                    Log(String.Format("Required[{0}] = {1}", i, req[i]));
                    if (String.Compare(req[i], "RequireSurchargeDecision", true ) == 0)
                    {
                        Dictionary<string, string> info = new Dictionary<string, string>();
                        info["RequireSurchargeDecision"] = "Decline";
                        var binLookupOutcome = paymentResults.GetExtraChargeLookupOutcome();
                        if (binLookupOutcome != null && binLookupOutcome.allowExtraCharge)
                        {
                            Log(String.Format("ExtraChargeLookupOutcome allowCreditSurcharge {0}", binLookupOutcome.allowExtraCharge));

                            string caption = "Surcharge Decision";
                            double totalAmount = ((double)binLookupOutcome.totalAmount.getAmountInMinorUnits()) / 100;
                            double surchargeAmount = ((double)binLookupOutcome.extraChargeAmount.getAmountInMinorUnits()) / 100;
                            string message = "Total Amount: $" + totalAmount.ToString("F", CultureInfo.InvariantCulture) + "\n\n"
                                + "You will be assessed a " + binLookupOutcome.extraChargePercent.ToString("F", CultureInfo.InvariantCulture)
                                + "% surcharge amount of $" + surchargeAmount.ToString("F", CultureInfo.InvariantCulture)
                                + " for using a credit card";
                            MessageBoxButton buttons = MessageBoxButton.OKCancel;

                            // Displays the MessageBox.
                            MessageBoxResult result = MessageBox.Show(message, caption, buttons);
                            info["RequireSurchargeDecision"] = result == MessageBoxResult.OK ? "Accept" : "Decline";
                        }
                        m_CWS.StartContinuePaymentTransaction(m_PaymentGatewayId, chanId, info, MyNotifyCWSEvent, MyPaymentComplete);
                    }
                    else if (String.Compare(req[i], "RequireSurchargeConfirmation", true) == 0)
                    {
                        Dictionary<string, string> info = new Dictionary<string, string>();
                        info["RequireSurchargeConfirmation"] = "Accept";
                        var binLookupOutcome = paymentResults.GetExtraChargeLookupOutcome();
                        if (binLookupOutcome != null && binLookupOutcome.allowExtraCharge)
                        {
                            Log(String.Format("ExtraChargeLookupOutcome allowCreditSurcharge {0}", binLookupOutcome.allowExtraCharge));

                            string caption = "Surcharge Confirmation";
                            double totalAmount = ((double)binLookupOutcome.totalAmount.getAmountInMinorUnits()) / 100;
                      
                            string message = "Total Amount: $" + totalAmount.ToString("F", CultureInfo.InvariantCulture) + "\n\n"
                                + "You will be assessed a " + binLookupOutcome.extraChargePercent.ToString("F", CultureInfo.InvariantCulture)
                                + "% surcharge for using a credit card";
                            MessageBoxButton buttons = MessageBoxButton.YesNoCancel;

                            // Displays the MessageBox.
                            MessageBoxResult result = MessageBox.Show(message, caption, buttons);
                            info["RequireSurchargeConfirmation"] = result == MessageBoxResult.Yes ? "Accept" : 
                                (result == MessageBoxResult.No ? "Change_Card" : "Decline");
                        }
                        m_CWS.StartContinuePaymentTransaction(m_PaymentGatewayId, chanId, info, MyNotifyCWSEvent, MyPaymentComplete);
                    }
                    else if (String.Compare(req[i], "RequireServiceFeePercentageConfirmation", true) == 0)
                    {
                        Dictionary<string, string> info = new Dictionary<string, string>();
                        info["RequireServiceFeePercentageConfirmation"] = "Accept";
                        var serviceFeeLookupOutcome = paymentResults.GetExtraChargeLookupOutcome();
                        if (serviceFeeLookupOutcome != null && serviceFeeLookupOutcome.allowExtraCharge)
                        {
                            Log(String.Format("ExtraChargeLookupOutcome allowExtraCharge {0}", serviceFeeLookupOutcome.allowExtraCharge));

                            string caption = "Service Fee Confirmation";
                            double totalAmount = ((double)serviceFeeLookupOutcome.totalAmount.getAmountInMinorUnits()) / 100;
                            string message = "Total Amount: $" + totalAmount.ToString("F", CultureInfo.InvariantCulture) + "\n\n"
                                + "You will be assessed a " + serviceFeeLookupOutcome.extraChargePercent.ToString("F", CultureInfo.InvariantCulture)
                                + "% service fee";
                            MessageBoxButton buttons = MessageBoxButton.YesNo;

                            // Displays the MessageBox.
                            // Service fee DOES NOT support change card
                            MessageBoxResult result = MessageBox.Show(message, caption, buttons);
                            info["RequireServiceFeePercentageConfirmation"] = result == MessageBoxResult.Yes ? "Accept" : "Decline";
                        }
                        m_CWS.StartContinuePaymentTransaction(m_PaymentGatewayId, chanId, info, MyNotifyCWSEvent, MyPaymentComplete);
                    }
                    else if (String.Compare(req[i], "RequireServiceFeeAmountConfirmation", true) == 0)
                    {
                        Dictionary<string, string> info = new Dictionary<string, string>();
                        info["RequireServiceFeeAmountConfirmation"] = "Accept";
                        var serviceFeeLookupOutcome = paymentResults.GetExtraChargeLookupOutcome();
                        if (serviceFeeLookupOutcome != null && serviceFeeLookupOutcome.allowExtraCharge)
                        {
                            Log(String.Format("ExtraChargeLookupOutcome allowExtraCharge {0}", serviceFeeLookupOutcome.allowExtraCharge));

                            string caption = "Service Fee Confirmation";
                            double totalAmount = ((double)serviceFeeLookupOutcome.totalAmount.getAmountInMinorUnits()) / 100;
                            double surchargeAmount = ((double)serviceFeeLookupOutcome.extraChargeAmount.getAmountInMinorUnits()) / 100;
                            string message = "Total Amount: $" + totalAmount.ToString("F", CultureInfo.InvariantCulture) + "\n\n"
                                + "You will be assessed a flat service fee amount of $" + surchargeAmount.ToString("F", CultureInfo.InvariantCulture);
                            MessageBoxButton buttons = MessageBoxButton.YesNo;

                            // Displays the MessageBox.
                            // Service fee DOES NOT support change card
                            MessageBoxResult result = MessageBox.Show(message, caption, buttons);
                            info["RequireServiceFeeAmountConfirmation"] = result == MessageBoxResult.Yes ? "Accept" : "Decline";
                        }
                        m_CWS.StartContinuePaymentTransaction(m_PaymentGatewayId, chanId, info, MyNotifyCWSEvent, MyPaymentComplete);
                    }
                    else if (String.Compare(req[i], "RequirePaymentAccountSelection", true) == 0)
                    {
                        Dictionary<string, string> info = new Dictionary<string, string>();
                        PaymentAndAccountType[] paymentAndAccountTypeSelectionList = paymentResults.GetPaymentAndAccountTypeSelectionList();

                        Application.Current.Dispatcher.Invoke((Action)delegate
                        {
                            PaymentAndAccountTypeSelectionDialog paymentAndAccountTypeSelection = 
                            new PaymentAndAccountTypeSelectionDialog(paymentAndAccountTypeSelectionList);
                            paymentAndAccountTypeSelection.Owner = this;
                            paymentAndAccountTypeSelection.ShowDialog();

                         
                            info["PaymentAndAccountTypeSelected"] = paymentAndAccountTypeSelection.SelectedPaymentAndAccountType.ToString();
                            m_CWS.StartContinuePaymentTransaction(m_PaymentGatewayId, chanId, info, MyNotifyCWSEvent, MyPaymentComplete);
                        });
                    }
                    else if (String.Compare(req[i], "HandleBinLookupResult", true) == 0)
                    {
                        HandleBinLookupResultInfo binLookupResultInfo = null;
                        BinLookupResult binLookupResult = paymentResults.GetBinLookupResult();
                        if (binLookupResult.isHsaCard)
                        {
                            Application.Current.Dispatcher.Invoke(delegate
                            {
                                String currencyString = cb_Currency.SelectedItem.ToString();
                                HealthcareAmountsDialog dlg = new HealthcareAmountsDialog(currencyString);
                                dlg.Owner = this;
                                dlg.ShowDialog();

                                if (dlg.CancelTransaction)
                                {
                                    m_CWS.StartCancelPaymentTransaction(m_PaymentGatewayId, chanId, MyNotifyCWSEvent, MyPaymentComplete);
                                    return;
                                }

                                if (dlg.DoHealthcareAmounts())
                                {
                                    HealthcareAmount healthcareAmount = new HealthcareAmount();
                                    healthcareAmount.overTheCounterAmount = dlg.otcAmount;
                                    healthcareAmount.prescriptionAmount = dlg.prescriptionAmount;
                                    healthcareAmount.clinicAmount = dlg.clinicAmount;
                                    healthcareAmount.dentalAmount = dlg.dentalAmount;
                                    healthcareAmount.visionAmount = dlg.visionAmount;
                                    healthcareAmount.transitAmount = dlg.transitAmount;

                                    binLookupResultInfo = new HandleBinLookupResultInfo();
                                    binLookupResultInfo.healthcareAmount = healthcareAmount;
                                }

                                m_CWS.StartHandleBinLookupResult(m_PaymentGatewayId, chanId, binLookupResultInfo, MyNotifyCWSEvent, MyPaymentComplete);
                            });
                        }
                        else
                        {
                            m_CWS.StartHandleBinLookupResult(m_PaymentGatewayId, chanId, binLookupResultInfo, MyNotifyCWSEvent, MyPaymentComplete);
                        }
                    }
                }
            }
            ShowResults(paymentResults);
            
            FillTranType();
        }

        public void CalculateExtraChargeCompleteCallback(PaymentTransactionResults paymentResults)
        {
            Log("----------- CalculateExtraCharge Complete ------------");
            String chanId = paymentResults.GetChannelId();
            SetUIChanId(chanId);
            PaymentTransactionData paymentData = paymentResults.PaymentTransactionData;
            if (null != paymentData)
            {
                m_TransData = paymentData;

                if (null != paymentData.id)
                {
                    Log("id", paymentData.id);
                }


                if (null != paymentData.errors)
                {
                    for (int i = 0; i < paymentData.errors.Length; i++)
                    {
                        Log(String.Format("Error[{0}] =  {1}", i, paymentData.errors[i]));
                    }
                }
            }

            var serviceFeeLookupOutcome = paymentResults.GetExtraChargeLookupOutcome();
            if (serviceFeeLookupOutcome != null)
            {
                Log("Extra Charge Type: " + serviceFeeLookupOutcome.extraChargeLookupType);
                Log("Extra Charge Percentage", serviceFeeLookupOutcome.extraChargePercent.ToString("F", CultureInfo.InvariantCulture));
                displayAmount("Total Amount", serviceFeeLookupOutcome.totalAmount);
                displayAmount("Base Amount", serviceFeeLookupOutcome.baseAmount);
                displayAmount("Extra Charge Amount", serviceFeeLookupOutcome.extraChargeAmount);
                ShowResults(paymentResults);
            }
            else
            {
                Log("Extra Charge outcome is null");
            }
        }

        private void displayAmount(String type, Money amount)
        {
            if (amount != null)
            {
                double totalAmount = ((double)amount.getAmountInMinorUnits()) / 100;
                Log(type, totalAmount.ToString("F", CultureInfo.InvariantCulture));
            }
        }

        private String LineBreak(String input)
        {
            int MAX = 150;
            if (input.Length <= MAX)
                return input;
            StringBuilder sb = new StringBuilder();
            sb.Append(input.Substring(0, MAX));
            input = input.Substring(MAX);
            while (input.Length >= MAX)
            {
                sb.Append("\n");
                sb.Append(input.Substring(0, MAX));
                input = input.Substring(MAX);
            }
            sb.Append("\n");
            sb.Append(input);
            return sb.ToString();
        }

        private void ShowResults(AResults sr)
        {
            Log(String.Format("Completed {0}", sr.Completed ? "TRUE" : "FALSE"));
            Log(String.Format("status {0}", sr.StatusDetails));


            Dispatcher.Invoke((Action)delegate
            {
                cb_aidSelection.Items.Clear();

                cb_paymentTypeSelected.IsChecked = false;
                cb_paymentTypeSelection.Items.Clear();

                cb_accountTypeSelected.IsChecked = false;
                cb_accountTypeSelection.Items.Clear();

                if (sr.Completed)
                {
                    SetTransState(TransState.Idle);
                }
                else
                {
                    if (sr is PaymentTransactionResults)
                    {
                        EmvApplication[] emvApps = ((PaymentTransactionResults)sr).GetEmvApplicationSelectionList();
                        if (emvApps != null && emvApps.Length > 0)
                        {
                            foreach (EmvApplication emvApplication in emvApps)
                            {
                                cb_aidSelection.Items.Add(emvApplication);
                            }
                            cb_aidSelection.SelectedIndex = 0;
                        }

                        PaymentType[] paymentTypes = ((PaymentTransactionResults)sr).GetPaymentTypeSelectionList();
                        if (paymentTypes != null && paymentTypes.Length > 0)
                        {
                            foreach (PaymentType paymentType in paymentTypes)
                            {
                                cb_paymentTypeSelection.Items.Add(paymentType);
                            }
                            cb_paymentTypeSelection.SelectedIndex = 0;
                        }

                        AccountType[] accountTypes = ((PaymentTransactionResults)sr).GetAccountTypeSelectionList();
                        if (accountTypes != null && accountTypes.Length > 0)
                        {
                            foreach (AccountType accountType in accountTypes)
                            {
                                cb_accountTypeSelection.Items.Add(accountType);
                            }
                            cb_accountTypeSelection.SelectedIndex = 0;
                        }
                    }
                    SetTransState(TransState.Paused);
                }

                //if (null == sr)
                //    l_JsonResp.Content = "Null StatusResponse";
                //else
                //{
                //    String broken = LineBreak(sr.RawJSON);
                //    l_JsonResp.Content = broken;
                //    
                //}
            });
        }

        /*
         * MyNotifyCWSEvent - called when interesting things happen.
         * Just show them in the UI
         */
        public void MyNotifyCWSEvent(CWSEvent cwsevent)
        {
            if (m_IsAsync)
            {
                DateTime now = CWSStringToDateTime(cwsevent.timeStamp);
                DateTime local = now.ToLocalTime();
                Log(String.Format("MyNotifyCWSEvent {0} at {1} with chanId = {2}", cwsevent.statusDetails, local.ToString(), cwsevent.chanId));
                if (!String.IsNullOrEmpty(cwsevent.chanId))
                {
                    m_ChanId = cwsevent.chanId;
                }
            }
        }

        /* Date Handling */
        private static readonly DateTime Jan1st1970 = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);

        public static DateTime CWSStringToDateTime(String datelong)
        {
            return CWSLongToDateTime(long.Parse(datelong));
        }

        public static DateTime CWSLongToDateTime(long javaLong)
        {
            TimeSpan ss = TimeSpan.FromMilliseconds(javaLong);
            DateTime Jan1st1970 = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);
            DateTime ddd = Jan1st1970.Add(ss);
            DateTime final = ddd.ToUniversalTime();

            return final;
        }

        private long ParseLong(string somevalue)
        {
            long rv = -1;
            try
            {
                rv = long.Parse(somevalue);
            }
            catch (Exception)
            {

            }
            if (-1 == rv)
                MessageBox.Show(String.Format("Invalid amount {0}", somevalue), "ERROR");
            return rv;
        }

        private void doPreAuthUpdate(TransSearchDlg dlg, string transactionType)
        {
            PaymentArgs bea = new PaymentArgs();
            bea.transactionType = transactionType;
            bea.paymentGatewayId = m_PaymentGatewayId;
            bea.originalTransId = dlg.getOrigTransId();
            bea.tenderType = dlg.getOrigTender();
            bea.oarData = dlg.getOarData();
            bea.ps2000Data = dlg.getPs2000Data();
            SetAdditionalTransactionPropertiesIfProvided(bea);

            String origCurrency = dlg.getOrigCurrency();
            String amount = dlg.getPreAuthAmount();
            if ((null != origCurrency) && (null != amount) && (0 < amount.Length))
            {
                bea.baseTransactionAmount = new Money(Money.CurrencyCodefromString(origCurrency), ParseLong(amount));
            }
            m_CWS.StartPaymentTransaction(bea, MyNotifyCWSEvent, MyPaymentComplete);
        }

        /*
         * Fire up the Search Transaction dialog.
         */
        private void b_SearchTransaction_Click(object sender, RoutedEventArgs e)
        {
            TransSearchDlg dlg = new TransSearchDlg(m_CWS, m_PaymentGatewayId);
            dlg.Owner = this;
            dlg.ShowDialog();

            /*
             * The user can select a transaction, then request an operation on that 
             * transaction.  The work gets done here.
             * See if they want us to do something, and verify parameters are ok
             */
            String origTransId, origTender, origCardType, origAmount, origCurrency = "", origDate, tipAmountString;
            if ((dlg.DoRefund())
             && (null != (origTransId = dlg.getOrigTransId()))
             && (null != (origCurrency = dlg.getOrigCurrency()))
             && (null != (origCardType = dlg.getOrigCardType()))
             && (null != (origAmount = dlg.getRefundableAmount()))
             && (null != (origTender = dlg.getOrigTender())))
            {
                string note = dlg.getNote();
                if (null == note)
                    note = "";
                Log("------------- Linked Refund ------------");
                long finalAmount = ParseLong(origAmount);
                Money amount = new Money(Money.CurrencyCodefromString(origCurrency), finalAmount);

                m_CWS.StartLinkedRefund(m_PaymentGatewayId, origTransId, origTender, origCardType, amount, note, MyNotifyCWSEvent, MyPaymentComplete);
            }
            else if ((dlg.DoVoid())
             && (null != (origTransId = dlg.getOrigTransId()))
             && (null != (origCurrency = dlg.getOrigCurrency()))
             && (null != (origAmount = dlg.getOrigAmountMinusSurcharge()))
             && (null != (origTender = dlg.getOrigTender())))
            {
                Log("------------- Void ------------");
                PaymentArgs bea = new PaymentArgs();
                bea.transactionType = TransactionType.VOID;
                bea.paymentGatewayId = m_PaymentGatewayId;
                bea.originalTransId = origTransId;
                bea.tenderType = dlg.getOrigTender();
                bea.baseTransactionAmount = new Money(Money.CurrencyCodefromString(origCurrency), origAmount);
                bea.oarData = dlg.getOarData();
                bea.ps2000Data = dlg.getPs2000Data();
                SetAdditionalTransactionPropertiesIfProvided(bea);
                m_CWS.StartPaymentTransaction(bea, MyNotifyCWSEvent, MyPaymentComplete);
            }
            if ((dlg.DoTipAdjust())
             && (null != (origTransId = dlg.getOrigTransId()))
             && (null != (origCurrency = dlg.getOrigCurrency()))
             && (null != (tipAmountString = dlg.getTipAmount())))
            {
                Log("------------- Tip Adjust ------------");
                long tipAmountLong = ParseLong(tipAmountString);
                Money tipAmount = new Money(Money.CurrencyCodefromString(origCurrency), tipAmountLong);

                string caption = "Tip Adjust Verification";
                string message = "Should this Tip Adjust transaction be authorized by the issuer?";
                MessageBoxButton buttons = MessageBoxButton.YesNo;
                // Displays the MessageBox.
                MessageBoxResult result = MessageBox.Show(message, caption, buttons);
                m_CWS.StartTipAdjust(m_PaymentGatewayId, origTransId, tipAmount, result == MessageBoxResult.Yes, MyNotifyCWSEvent, MyPaymentComplete);
            }
            else if ((dlg.DoPrint())
             && (null != (origTransId = dlg.getOrigTransId()))
             && (null != (origDate = dlg.getOrigDate())))
            {
                Log("------------ Print Transaction -----------");
                m_PrintPaymentArgs.transactionId = origTransId;
                m_PrintPaymentArgs.utcDate = origDate;
                m_CWS.StartPrintPaymentReceipt(m_PrintPaymentArgs, MyNotifyCWSEvent, MyPaymentComplete);
            }
            else if (dlg.DoCustomPrint())
            {
                Log("------------- Custom Print Transaction  ------------");
                Transaction tran = dlg.getPrintTran();
                CustomPrint(tran, true, null);
            }
            else if (dlg.DoPreAuthIncrement())
            {
                Log("------------- Pre Auth Increment  ------------");
                doPreAuthUpdate(dlg, TransactionType.PRE_AUTH_INCREMENT);
            }
            else if (dlg.DoPreAuthComplete())
            {
                Log("------------- Pre Auth Complete  ------------");
                doPreAuthUpdate(dlg, TransactionType.PRE_AUTH_COMPLETE);
            }
            else if (dlg.DoPreAuthDelete())
                //&& (null != (origTransId = dlg.getOrigTransId())))
            {
                Log("------------- Pre Auth Delete  ------------");
                doPreAuthUpdate(dlg, TransactionType.PRE_AUTH_DELETE);
            }
        }

        /* Original quick search - no longer used */
        private void o_SearchTransactionClick(object sender, RoutedEventArgs e)
        {
            TransactionSearchArgs tsa = new TransactionSearchArgs();
            tsa.paymentGatewayId = m_PaymentGatewayId;
            tsa.last4CC = "6919";
            DateTime today = System.DateTime.Now;
            TimeSpan threeDays = new TimeSpan(3, 0, 0, 0);
            DateTime threeDaysAgo = today.Subtract(threeDays);

            tsa.beginDate = threeDaysAgo.ToString("yyyyMMdd");
            tsa.endDate = today.ToString("yyyyMMdd");

            Log(String.Format("Search:  last4 {0}, dates {1} - {2}", tsa.last4CC, tsa.beginDate, tsa.endDate));

            bool? check = cb_Async.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                m_CWS.StartTransactionSearch(tsa, MyNotifyCWSEvent, MyTransactionSearchComplete);
            }
            else
            {
                TransactionSearchResults tsr = m_CWS.TransactionSearch(tsa, null);
                MyTransactionSearchComplete(tsr);
            }
        }

        void MyTransactionSearchComplete(TransactionSearchResults tsr)
        {
            TransactionSearchData tsd = tsr.getTransactionSearchData();
            Log(String.Format("SearchComplete {0}", tsd.status));
            Transaction [] trans = tsd.transactions;
            if (null != trans)
            {
                foreach (Transaction tran in trans)
                {
                    Log(String.Format("{0},{1},{2},{3},{4},{5},{6},{7},{8}",
                        tran.id, tran.transactionType, tran.tenderType, tran.date, tran.amount, tran.authCode, 
                        tran.ivuControlNumber, tran.ivuMunicipalTax, tran.ivuStateTax));
                }
            }
            String[] msgs = tsd.messages;
            if (null != msgs)
            {
                foreach (String msg in msgs)
                {
                    Log(msg);
                }
            }
        }

        private void b_PrintReceipt_Click(object sender, RoutedEventArgs e)
        {
            Log("----------- Print Receipt ------------");
            m_PrintPaymentArgs.transactionId = null;
            bool ? check = cb_Async.IsChecked;
            if ((check.HasValue) && (check.Value))
                m_CWS.StartPrintPaymentReceipt(m_PrintPaymentArgs, MyNotifyCWSEvent, MyPaymentComplete);
            else
            {
                PaymentTransactionResults tr = m_CWS.PrintPaymentReceipt(m_PrintPaymentArgs, null);
                MyPaymentComplete(tr);
            }
        }

        private void b_EmailReceipt_Click(object sender, RoutedEventArgs e)
        {
            Log("----------- Email Receipt ------------");
            EmailPaymentArgs emailargs = new EmailPaymentArgs();

            /* copy items from print args */
            emailargs.paymentGatewayId = m_PrintPaymentArgs.paymentGatewayId;
            if (null != m_PrintPaymentArgs.accountInfo)
            {
                AccountInfo accountInfo = new AccountInfo();
                accountInfo.name = m_PrintPaymentArgs.accountInfo.name;
                accountInfo.currencyCode = m_PrintPaymentArgs.accountInfo.currencyCode;
                accountInfo.businessEmail = m_PrintPaymentArgs.accountInfo.businessEmail;
                accountInfo.address1 = m_PrintPaymentArgs.accountInfo.address1;
                accountInfo.city = m_PrintPaymentArgs.accountInfo.city;
                accountInfo.stateProvince = m_PrintPaymentArgs.accountInfo.stateProvince;
                accountInfo.postalCode = m_PrintPaymentArgs.accountInfo.postalCode;
                emailargs.accountInfo = accountInfo;
            }
            emailargs.receiptTerms = m_PrintPaymentArgs.receiptTerms;

            /* Add email address */

            emailargs.emailAddress = tb_EmailAddress.Text;
            emailargs.saveEmail = (cb_SaveEmail.IsChecked.HasValue && cb_SaveEmail.IsChecked.Value);

            bool ? check = cb_Async.IsChecked;
            if ((check.HasValue) && (check.Value))
                m_CWS.StartEmailPaymentReceipt(emailargs, MyNotifyCWSEvent, MyPaymentComplete);
            else
            {
                PaymentTransactionResults tr = m_CWS.EmailPaymentReceipt(emailargs, null);
                MyPaymentComplete(tr);
            }
        }

        private void b_SaveReceipt_Click(object sender, RoutedEventArgs e)
        {
            Log("----------- Save Receipt ------------");
            bool? check = cb_Async.IsChecked;
            if ((check.HasValue) && (check.Value))
                m_CWS.StartFilePaymentReceipt(m_PrintPaymentArgs, MyNotifyCWSEvent, MyPaymentComplete);
            else
            {
                PaymentTransactionResults tr = m_CWS.FilePaymentReceipt(m_PrintPaymentArgs, null);
                MyPaymentComplete(tr);
            }
        }

        private void b_TLVUpdate_Click(object sender, RoutedEventArgs e)
        {
            Log("----------- TLVUpdate ------------");
            Dictionary<String, String> tlvs = new Dictionary<String, String>();

            tlvs.Add("999", "777");         /* This one allowed (on white list) */
            // tlvs.Add("Name", "Value");   /* this one not allowed */

            TLVArgs args = new TLVArgs();
            args.paymentGatewayId = m_PaymentGatewayId;
            args.tlvList = tlvs;

            bool? check = cb_Async.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                m_CWS.StartTLVUpdate(args, MyNotifyCWSEvent, MyPaymentComplete);
            }
            else
            {
                PaymentTransactionResults sr = m_CWS.TLVUpdate(args, null);
                MyPaymentComplete(sr);
            }
        }

        private void b_ValidateAccount_Click(object sender, RoutedEventArgs e)
        {
            Log("----------- Validate Account ------------");
            bool? check = cb_Async.IsChecked;
            if ((check.HasValue) && (check.Value))
            {
                m_CWS.StartValidateAccount(m_PaymentGatewayId, MyNotifyCWSEvent, MyPaymentComplete);
            }
            else
            {
                PaymentTransactionResults sr = m_CWS.ValidateAccount(m_PaymentGatewayId, null);
                MyPaymentComplete(sr);
            }
        }

        private String CleanForJSON(String inStr)
        {
            String t1 = inStr.Replace("\\", "\\\\");
            String t2 = t1.Replace("\"", "\\\"");
            return t2;
        }

        private string GetFullPath(string fname)
        {
            if (':' == fname[1])
                return fname;
            string path = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location) + "\\Resources\\" + fname;
            return path;
        }

        private string LoadJPG()
        {
            string path = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location) + "\\Resources\\logo_CV.jpg";
            
            string b64 = null;
            try
            {
                b64 = Convert.ToBase64String(File.ReadAllBytes(path));
            }
            catch (Exception)
            {
            }
            return b64;
        }

        private void SetTransState(TransState state)
        {
            m_TransState = state;
            if (TransState.InProgress == state)
            {
                b_Transaction.IsEnabled = false;

                b_Cancel.IsEnabled = true;
                b_ContinueCardData.IsEnabled = false;

                b_CustomPrint.IsEnabled = false;

                b_PrintReceipt.IsEnabled = false;
                b_EmailReceipt.IsEnabled = false;
                b_SaveReceipt.IsEnabled = false;
            }
            else if (TransState.Paused == state)
            {
                b_Transaction.IsEnabled = false;

                b_Cancel.IsEnabled = true;
                b_ContinueCardData.IsEnabled = true;

                b_CustomPrint.IsEnabled = false;

                b_PrintReceipt.IsEnabled = false;
                b_EmailReceipt.IsEnabled = false;
                b_SaveReceipt.IsEnabled = false;

            }
            else   /* Idle */
            {
                b_Transaction.IsEnabled = true;

                b_Cancel.IsEnabled = true;
                b_ContinueCardData.IsEnabled = false;

                if ((null != m_OriginalTransId) && (0 < m_OriginalTransId.Length))
                {
                    if (null != m_TransData)
                        b_CustomPrint.IsEnabled = true;
                    else
                        b_CustomPrint.IsEnabled = false;
                    b_PrintReceipt.IsEnabled = true;
                    b_EmailReceipt.IsEnabled = true;
                    b_SaveReceipt.IsEnabled = true;
                }
                else
                {
                    b_CustomPrint.IsEnabled = false;
                    b_PrintReceipt.IsEnabled = false;
                    b_EmailReceipt.IsEnabled = false;
                    b_SaveReceipt.IsEnabled = false;
                }
            }
        }

        private void b_CustomPrint_Click(object sender, RoutedEventArgs e)
        {
            if (null != m_TransData)
            {
                CustomPrint(m_TransData, false, m_TransData);
            }
        }

        /*
         * CustomPrint - Build up the complete text of a receipt, including
         * the markup necessary to print our bitmap and signature.
         */
        private void CustomPrint(Transaction transData, Boolean isReprint, PaymentTransactionData ptd)
        {
            {
                string b64 = LoadJPG();
                
                StringBuilder text = new StringBuilder();
                text.Append(PrintControl.NEWLINE);
                text.Append(PrintControl.CENTER);
                text.Append(PrintControl.BOLD_ON);
                text.Append("Pete's Electronics <c>");   /* Made up company name */
                text.Append(PrintControl.NEWLINE);
                text.Append(PrintControl.BOLD_OFF);

                if (null != b64)
                {
                    /* Surround the base 64 encoded JPEG */
                    text.Append(PrintControl.BASE64);
                    text.Append(b64);
                    text.Append(PrintControl.BASE64);
                }
                text.Append(PrintControl.NEWLINE);
                text.Append(PrintControl.LEFT);
                text.Append("Date: ");
                text.Append(transData.date);
                text.Append(PrintControl.NEWLINE);
                if (isReprint)
                {
                    text.Append(PrintControl.NEWLINE);
                    text.Append("***** Reprint *****");
                    text.Append(PrintControl.NEWLINE);
                }
                text.Append(PrintControl.NEWLINE);
                /* Start filling in the receipt with returned info */

                text.Append(String.Format("Transaction Type: {0}", transData.transactionType));
                text.Append(PrintControl.NEWLINE);
                text.Append(PrintControl.NEWLINE);

                if ((null != transData.maskedPan) && (0 < transData.maskedPan.Length))
                {
                    text.Append(String.Format("Card Number: {0}", transData.maskedPan));
                    text.Append(PrintControl.NEWLINE);
                }
                if (null != transData.iccMode)
                {
                    text.Append(String.Format("Entry Method: {0}", transData.iccMode));
                    text.Append(PrintControl.NEWLINE);
                }
                if (null != transData.cardScheme)
                {
                    text.Append(String.Format("Card Type: {0}", transData.cardScheme));
                    text.Append(PrintControl.NEWLINE);
                }
                if (null != transData.iccAppName)
                {
                    text.Append(String.Format("Application: {0}", transData.iccAppName));
                    text.Append(PrintControl.NEWLINE);
                }
                text.Append(String.Format("Tender Type: {0}", transData.tenderType));
                text.Append(PrintControl.NEWLINE);
                text.Append(String.Format("Approval Code: {0}", transData.authCode));
                //if (null != transData.iccCardType)
                //{
                //    text.Append(PrintControl.NEWLINE);
                //    text.Append(String.Format("iccCardType: {0}", transData.iccCardType));
                //}
                if (null != transData.iccCvmr)
                {
                    text.Append(PrintControl.NEWLINE);
                    text.Append(String.Format("Verification Method: {0}", transData.iccCvmr));
                }
                if (null != transData.result)
                {
                    text.Append(PrintControl.NEWLINE);
                    text.Append(String.Format("Result Message: {0}", transData.result));
                }
                text.Append(PrintControl.NEWLINE);
                text.Append(String.Format("Trans Ref: {0}", transData.id));
                text.Append(PrintControl.NEWLINE);
                if (null != transData.iccAid)
                {
                    text.Append(String.Format("AID: {0}", transData.iccAid));
                    text.Append(PrintControl.NEWLINE);
                }
                if (null != transData.iccTvr)
                {
                    text.Append(String.Format("TVR: {0}", transData.iccTvr));
                    text.Append(PrintControl.NEWLINE);
                }
                if (null != transData.iccTsi)
                {
                    text.Append(String.Format("TSI: {0}", transData.iccTsi));
                    text.Append(PrintControl.NEWLINE);
                }

                if (null != transData.iccAc)
                {
                    text.Append(String.Format("AC: {0}", transData.iccAc));
                    text.Append(PrintControl.NEWLINE);
                }
                if (null != transData.iccArc)
                {
                    text.Append(String.Format("ARC: {0}", transData.iccArc));
                    text.Append(PrintControl.NEWLINE);
                }
                if (null != transData.iccAtc)
                {
                    text.Append(String.Format("ATC: {0}", transData.iccAtc));
                    text.Append(PrintControl.NEWLINE);
                }
                if (null != transData.iccCsn)
                {
                    text.Append(String.Format("CSN: {0}", transData.iccCsn));
                    text.Append(PrintControl.NEWLINE);
                }
                if (null != transData.iccIad)
                {
                    text.Append(String.Format("IAD: {0}", transData.iccIad));
                    text.Append(PrintControl.NEWLINE);
                }
                if (null != transData.ivuControlNumber)
                {
                    text.Append(String.Format("IVU Control Number: {0}", transData.ivuControlNumber));
                    text.Append(PrintControl.NEWLINE);
                }
                text.Append(PrintControl.LEFT);
                AddLine(text);
                text.Append(PrintControl.NEWLINE);
                text.Append(PrintControl.RIGHT);

                /* Figure out base amount.  Amount from transData has tip and tax added to it */

                Money myAmount = transData.amount;
                Decimal dAmount = new Decimal(myAmount.value) / 100;
                Decimal dFinalAmount = dAmount;

                Decimal dTax = new Decimal(0);      /* Assume no tax */
                Money myTax = transData.tax;
                if (null != myTax)
                { 
                    dTax = new Decimal(myTax.value) / 100;
                    if (dZero < dTax)
                        dAmount -= dTax;            /* deduct from total */
                }

                Decimal dIvuMunicipalTax = new Decimal(0);      /* Assume no tax */
                Money ivuMunicipalTax = transData.ivuMunicipalTax;
                if (null != ivuMunicipalTax)
                {
                    dIvuMunicipalTax = new Decimal(ivuMunicipalTax.value) / 100;
                    if (dZero < dIvuMunicipalTax)
                        dAmount -= dIvuMunicipalTax;            /* deduct from total */
                }

                Decimal dIvuStateTax = new Decimal(0);      /* Assume no tax */
                Money ivuStateTax = transData.tax;
                if (null != ivuStateTax)
                {
                    dIvuStateTax = new Decimal(ivuStateTax.value) / 100;
                    if (dZero < dIvuStateTax)
                        dAmount -= dIvuStateTax;            /* deduct from total */
                }

                Decimal dTip = dZero;               /* Assume no tip */
                if (null != transData.gratuityAmount)
                {
                    Money ga = transData.gratuityAmount;
                    dTip = new Decimal(ga.value) / 100;
                    if (dZero < dTip)
                        dAmount -= dTip;            /* deduct from total */
                }
                Money myDiscount = transData.discount;
                Decimal dDiscount = dZero;          /* Assume no discount */
                if (null != myDiscount)
                {
                    dDiscount = new Decimal(myDiscount.value) / 100;
                    if (dZero < dDiscount)
                        dAmount += dDiscount;       /* add it back */
                }

                /* Now print the subtotal, tax, tip, discount and final total */
                String sAmount = String.Format("Subtotal: {0:C}", dAmount);
                text.Append(sAmount);
                text.Append(PrintControl.NEWLINE);

                if (dZero < dTax)
                {
                    String sTax = String.Format("Tax: {0:C}", dTax);
                    text.Append(sTax);
                    text.Append(PrintControl.NEWLINE);
                }
                if (dZero < dIvuMunicipalTax)
                {
                    String sTax = String.Format("IVU Municipal Tax: {0:C}", dIvuMunicipalTax);
                    text.Append(sTax);
                    text.Append(PrintControl.NEWLINE);
                }
                if (dZero < dIvuStateTax)
                {
                    String sTax = String.Format("IVU State Tax: {0:C}", dIvuStateTax);
                    text.Append(sTax);
                    text.Append(PrintControl.NEWLINE);
                }
                if (dZero < dTip)
                {
                    String sTip = String.Format("Tip: {0:C}", dTip);
                    text.Append(sTip);
                    text.Append(PrintControl.NEWLINE);
                }
                if (dZero < dDiscount)
                {
                    String sDiscount = String.Format("Discount: {0:C}", dDiscount);
                    text.Append(sDiscount);
                    text.Append(PrintControl.NEWLINE);
                }
                //{
                //    String myDiscount = tb_Discount.Text;
                //    Decimal dDiscount = Decimal.Parse(myDiscount.Trim()) / 100;
                //    if (dZero < dDiscount)
                //    {
                //        String sDiscount = String.Format("Discount: {0:C}", dDiscount);
                //        text.Append(sDiscount;
                //        text.Append(PrintControl.NEWLINE;

                //        dAmount -= dDiscount;
                //    }
                //}

                text.Append(PrintControl.BIG_ON);
                sAmount = String.Format("Total: {0:C}", dFinalAmount);
                text.Append(sAmount);
                text.Append(PrintControl.BIG_OFF);
                text.Append(PrintControl.NEWLINE);
                text.Append(PrintControl.LEFT);
                AddLine(text);
                if ((null != ptd) && (null != ptd.balanceDue) && (0 < ptd.balanceDue.value))
                {
                    text.Append(PrintControl.NEWLINE);
                    Decimal dBalance = new Decimal(ptd.balanceDue.value) / 100;
                    text.Append(PrintControl.BIG_ON);
                    text.Append(PrintControl.RIGHT);
                    sAmount = String.Format("Balance Due: {0:C}", dBalance);
                    text.Append(sAmount);

                    text.Append(PrintControl.BIG_OFF);
                    text.Append(PrintControl.NEWLINE);
                    text.Append(PrintControl.LEFT);
                    AddLine(text);
                }

                text.Append(PrintControl.NEWLINE);
                text.Append(PrintControl.LEFT);

                signatureBitmap sb = transData.signatureBitmap;
                if (null != sb)
                {
                    /* Surround the 3 byte ascii bitmap */
                    if (sb.format.Equals("JPG"))
                    {
                        text.Append(PrintControl.BASE64);
                        text.Append(CleanForJSON(sb.data));
                        text.Append(PrintControl.BASE64);
                    }
                    else
                    {
                        text.Append(PrintControl.SIGBIN);
                        text.Append(CleanForJSON(sb.data));
                        text.Append(PrintControl.SIGBIN);
                    }
                    text.Append(PrintControl.NEWLINE);
                    AddLine(text);
                }
                else
                {
                    text.Append(PrintControl.NEWLINE);       /* No signaure, give them space to sign */
                    text.Append(PrintControl.NEWLINE);
                    text.Append(PrintControl.NEWLINE);
                    text.Append("X_______________________________________");
                }
                text.Append(PrintControl.NEWLINE);
                text.Append(PrintControl.CENTER);
                text.Append("Printed by CWSTransApp");
                text.Append(PrintControl.NEWLINE);
                text.Append(PrintControl.NEWLINE);

                /* OK, send it off to be printed */

                bool? check = cb_Async.IsChecked;
                if ((check.HasValue) && (check.Value))
                {
                    m_CWS.StartPrintReceipt(text.ToString(), null, MyNotifyCWSEvent, MyPrintReceiptComplete);
                }
                else
                {
                    PrintReceiptResults rr = m_CWS.PrintReceipt(text.ToString(), null, null);
                    MyPrintReceiptComplete(rr);
                }
            }
        }

        void AddLine(StringBuilder text)
        {
            text.Append("______________________________________________");
        }

        private void cbox_TenderType_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
 
        }

        private void b_AccountInfo_Click(object sender, RoutedEventArgs e)
        {
        }

        private void b_DeviceConfig_Click(object sender, RoutedEventArgs e)
        {
            GetDeviceConfig();
        }

        private void b_GeneralSettings_Click(object sender, RoutedEventArgs e)
        {
            GeneralSettings dlg = new GeneralSettings();
            dlg.Owner = this;
            dlg.ShowDialog();
            if (dlg.DoSetTerminalDeviceConfiguration())
            {
                TerminalDeviceType deviceType = (TerminalDeviceType)Enum.Parse(typeof(TerminalDeviceType), dlg.getTerminalDeviceType());
                CatLevelIndicator catLevelIndicator = (CatLevelIndicator)Enum.Parse(typeof(CatLevelIndicator), dlg.getTerminalCatLevelIndicator());

                StartSetTerminalDeviceConfiguration(deviceType, catLevelIndicator);
            }
            else if (dlg.DoSetTerminalCardholderInputModeConfiguration())
            {
                TerminalCardholderInputMode cardholderInputMode = (TerminalCardholderInputMode)Enum.Parse(typeof(TerminalCardholderInputMode), dlg.getTerminalCardholderInputMode());

                StartSetTerminalCardholderInputModeConfiguration(cardholderInputMode);
            }
            else if (dlg.DoLanguageOverride())
            {
                StartOverrideDefaultTerminalLanguage(dlg.getLanguageCode(), dlg.getCountryCode());
            }
            else if (dlg.DoDebitNetworkOverride())
            {
                String visa = DebitNetworkChoice.toApiValue(dlg.getVisaDebitNetworkPreference());
                String mastercard = DebitNetworkChoice.toApiValue(dlg.getMastercardDebitNetworkPreference());
                String discover = DebitNetworkChoice.toApiValue(dlg.getDiscoverDebitNetworkPreference());
                StartOverrideDebitNetworkPreferences(visa, mastercard, discover);
            }
            else if (dlg.DoCvmOptionsOverride())
            {
                StartOverrideCvmOptions(dlg.getSignatureOption());
            }
            else if (dlg.DoSetLaneNumber())
            {
                StartSetTransactionLaneNumber(dlg.getLaneNumber());
            }
        }

        private void cb_Signature_Checked(object sender, RoutedEventArgs e)
        {
            cb_SignatureCancel.IsChecked = false;
        }

        private void cb_SignatureCancel_Checked(object sender, RoutedEventArgs e)
        {
            cb_Signature.IsChecked = false;
        }
    }
}


