﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;


using CWSWrapper;
namespace CWSTransApp
{
    /// <summary>
    /// Interaction logic for TransSearchDlg.xaml
    /// </summary>
    public partial class TransSearchDlg : Window
    {
        private bool last4;
        private bool first6;
        private bool complete;
        private CWS m_CWS;
        private String m_PaymentGatewayId;
        private bool m_DoRefund = false;
        private bool m_DoDelete = false;
        private bool m_DoPrint = false;
        private bool m_DoComplete = false;
        private bool m_DoIncrement = false;
        private bool m_DoVoid = false;
        private bool m_DoTipAdjust = false;
        private bool m_DoCustomPrint = false;
        private String m_OrigTransId = null;
        private String m_OrigTender = null;
        private String m_OrigCurrency = null;
        private String m_OrigCardType = null;
        private String m_OrigAmount = null;
        private String m_AmountMinusSurcharge = null;
        private String m_RefundableAmount = null;
        private String m_OrigTax = null;
        private String m_OrigTip = null;
        private String m_OrigDate = null;
        private String m_Note = null;
        private String m_PreAuthAmount = null;
        private String m_TipAmount = null;
        private String m_OarData = null;
        private String m_Ps2000Data = null;
        private int m_ResultCount;
        private Transaction m_PrintTran = null;

        public readonly static String CLOSE_REFUND = "Refund";
        public readonly static String CLOSE_DELETE = "Delete";
        public readonly static String CLOSE_COMPLETE = "Complete";

        public TransSearchDlg(CWS cws, String backendid)
        {
            InitializeComponent();

            m_CWS = cws;
            m_PaymentGatewayId = backendid;
            tb_CreditCard.Text = "6919";
            rb_Last4.IsChecked = true;
            rb_Complete.IsChecked = false;
            rb_First6.IsChecked = false;
            last4 = true;
            first6 = false;
            complete = false;
            m_ResultCount = 0;
            handleButtons();
            dp_StartDate.SelectedDate = DateTime.Today;
            dp_EndDate.SelectedDate = DateTime.Today;
        }

        private void rb_First6_Checked(object sender, RoutedEventArgs e)
        {
            rb_Complete.IsChecked = false;
            rb_Last4.IsChecked = false;
            last4 = false;
            first6 = true;
            complete = false;

        }

        private void rb_Last4_Checked(object sender, RoutedEventArgs e)
        {
            rb_Complete.IsChecked = false;
            rb_First6.IsChecked = false;
            last4 = true;
            first6 = false;
            complete = false;
        }

        private void rb_Complete_Checked(object sender, RoutedEventArgs e)
        {
            rb_First6.IsChecked = false;
            rb_Last4.IsChecked = false;
            last4 = false;
            first6 = false;
            complete = true;
        }

        private void clearResults()
        {
            List<LocalTran> items = new List<LocalTran>();
            lv_Results.ItemsSource = items;
        }
        /*
         * Do the search
         */
        private void b_Search_Click(object sender, RoutedEventArgs e)
        {
            clearResults();

            TransactionSearchArgs tsa = new TransactionSearchArgs();

            /* Figure out what search criteral was selected by the user */

            tsa.paymentGatewayId = m_PaymentGatewayId;
            if (complete)
                tsa.creditCard = tb_CreditCard.Text;
            else if (last4)
                tsa.last4CC = tb_CreditCard.Text;
            else if (first6)
                tsa.first6CC = tb_CreditCard.Text;

            DateTime ?beginDate = dp_StartDate.SelectedDate;
            if (beginDate.HasValue)
                tsa.beginDate = beginDate.Value.ToString("yyyyMMdd");

            DateTime ?endDate = dp_EndDate.SelectedDate;
            if (endDate.HasValue)
                tsa.endDate = endDate.Value.ToString("yyyyMMdd");

            tsa.note = tb_SearchNote.Text;
            tsa.merchantTransactionReference = tb_SearchMTR.Text;

            /* Do the search - synchronously */

            TransactionSearchResults tsr;
            using (new WaitCursor())
            {
                tsr = m_CWS.TransactionSearch(tsa, null);  /* Search */
            }

            StringBuilder sb = new StringBuilder();
            if (null != tsr)
            {
                TransactionSearchData tsd = tsr.getTransactionSearchData();
                if (null != tsd)
                {
                    /* Loop over the result array an load the list control */

                    m_ResultCount = 0;
                    Transaction[] trans = tsd.transactions;
                    if (null != trans)
                    {
                        List<LocalTran> items = new List<LocalTran>();

                        foreach (Transaction tran in trans)
                        {
                            if (null != tran)
                            {
                                m_ResultCount++;
                                var localTran = new LocalTran()
                                {
                                    Id = tran.id,
                                    TranType = tran.transactionType,
                                    Tender = tran.tenderType,
                                    CardEntryType = tran.cardEntryType,
                                    CardType = tran.cardType,
                                    MaskedPan = tran.maskedPan,
                                    Date = tran.date,
                                    Amount = tran.amount.getAmountInMinorUnits(),
                                    AuthCode = tran.authCode,
                                    Result = tran.result,
                                    Currency = tran.amount.currencyCode,
                                    Voidable = tran.voidable,
                                    TipAdjustable = tran.tipAdjustable,
                                    Refundable = tran.refundable,
                                    Tax = (null == tran.tax) ? "null" : tran.tax.getAmountInMinorUnits().ToString(),
                                    Tip = (null == tran.gratuityAmount) ? "null" : tran.gratuityAmount.getAmountInMinorUnits().ToString(),
                                    InvoiceNum = tran.invoiceNumber,
                                    LaneNum = tran.laneNumber,
                                    OarData = tran.oarData,
                                    Ps2000Data = tran.ps2000Data,

                                    IvuControlNumber = tran.ivuControlNumber,
                                    IvuMunicipalTax = (tran.ivuMunicipalTax == null) ? "null" : tran.ivuMunicipalTax.getAmountInMinorUnits().ToString(),
                                    IvuStateTax = (tran.ivuStateTax == null) ? "null" : tran.ivuStateTax.getAmountInMinorUnits().ToString()
                                };
                                localTran.AmountRefundable = (tran.amountRefundable != null) ?
                                    tran.amountRefundable.getAmountInMinorUnits() : 0L;

                                localTran.AmountRefunded = (tran.amountRefunded != null) ?
                                    tran.amountRefunded.getAmountInMinorUnits() : 0L;
                                localTran.SurchargeAmount = (tran.totalSurchargeAmount != null) ?
                                  tran.totalSurchargeAmount.getAmountInMinorUnits() : 0L;
                                localTran.SurchargeAmountRefunded = (tran.surchargeAmountRefunded != null) ?
                                  tran.surchargeAmountRefunded.getAmountInMinorUnits() : 0L;

                                localTran.ServiceFeeTxnId = tran.serviceFeeTransactionId;
                                if (!string.IsNullOrEmpty(localTran.ServiceFeeTxnId))
                                {
                                    localTran.ServiceFeeTxnApprovalCode = tran.serviceFeeTransactionApprovalCode;
                                    localTran.ServiceFeeTxnDate = tran.serviceFeeTransactionDate;
                                    localTran.ServiceFeeTxnAmount = tran.serviceFeeAmount.getAmountInMinorUnits().ToString();
                                    localTran.ServiceFeeTxnRefundedAmount = (tran.refundedServiceFeeAmount == null) ? "null" :
                                        tran.refundedServiceFeeAmount.getAmountInMinorUnits().ToString();
                                }

                                localTran.OtcAmount = (tran.overTheCounterAmount != null) ?
                                    tran.overTheCounterAmount.getAmountInMinorUnits() : 0L;
                                localTran.PrescriptionAmount = (tran.prescriptionAmount != null) ?
                                    tran.prescriptionAmount.getAmountInMinorUnits() : 0L;
                                localTran.ClinicAmount = (tran.clinicAmount != null) ?
                                    tran.clinicAmount.getAmountInMinorUnits() : 0L;
                                localTran.DentalAmount = (tran.dentalAmount != null) ?
                                    tran.dentalAmount.getAmountInMinorUnits() : 0L;
                                localTran.VisionAmount = (tran.visionAmount != null) ?
                                    tran.visionAmount.getAmountInMinorUnits() : 0L;
                                localTran.TransitAmount = (tran.transitAmount != null) ?
                                    tran.transitAmount.getAmountInMinorUnits() : 0L;

                                items.Add(localTran);
                            }
                        }
                        lv_Results.ItemsSource = items;
                    }
                    sb.Append(tsd.status);
                    sb.Append(String.Format(", {0} items", m_ResultCount));
                    String[] msgs = tsd.messages;
                    if (null != msgs)
                    {
                        foreach (String msg in msgs)
                        {
                            sb.Append(", ");
                            sb.Append(msg);
                        }
                    }
                }
                else
                    sb.Append("****** No TransactionSearchData! ******");
            }
            else
                sb.Append("****** No TransactionSearchResults! ******");
            l_Message.Content = sb.ToString();
        }
        private void Log(String msg)
        {
            Dispatcher.Invoke((Action)delegate
            {
                l_Message.Content = msg;
            });
        }
        private void ShowSearchResult(Transaction tran)
        {
            String msg = (new CWSWrapper.JsonHandler()).Serialize<Transaction>(tran);
            MessageBox.Show(msg, "INFO");
        }
        private void b_Info_Click(object sender, RoutedEventArgs e)
        {
            ShowOrPrint(true);
        }
        private Transaction ShowOrPrint(Boolean show)
        {
            Transaction rv = null;
            StringBuilder sb = new StringBuilder();
            if ((0 < m_ResultCount)
             && (null != lv_Results.SelectedItems)
             && (0 < lv_Results.SelectedItems.Count))
            {
                LocalTran locTran = (LocalTran)lv_Results.SelectedItems[0];
                TransactionSearchArgs tsa = new TransactionSearchArgs();

                tsa.paymentGatewayId = m_PaymentGatewayId;
                tsa.transId = locTran.Id ;
                tsa.utcDate = locTran.Date ;

                TransactionSearchResults tsr;
                using (new WaitCursor())
                {
                    tsr = m_CWS.TransactionSearch(tsa, null);  /* Search */
                }

                TransactionSearchData tsd = tsr.getTransactionSearchData();
                if (null != tsd)
                {
                    Transaction[] trans = tsd.transactions;
                    if ((null != trans) && (0 < trans.Length))
                    {
                        if (show)
                            ShowSearchResult(trans[0]);
                        else
                            rv = trans[0];
                    }
                    else
                        sb.Append("****** No Transactions ******");
                }
                else
                    sb.Append("****** No TransactionSearchData ******");
            }
            else
                sb.Append("****** No transaction selected ******");
            l_Message.Content = sb.ToString();
            return rv; 
        }
        /*
         * The dialog is closing.  Save off the selected transaction data
         * because main may call back for it.
         */
        private void closeDlg()
        {
            LocalTran locTran = (LocalTran)lv_Results.SelectedItems[0];
            m_OrigTransId = locTran.Id;
            m_OrigDate = locTran.Date;
            m_OrigTender = locTran.Tender;
            m_OrigCardType = locTran.CardType;
            m_OrigCurrency = locTran.Currency;
            m_OrigAmount = locTran.Amount.ToString();
            m_AmountMinusSurcharge = (locTran.Amount - locTran.SurchargeAmount).ToString();
            m_RefundableAmount = locTran.AmountRefundable.ToString();
            m_OrigTip = locTran.Tip.ToString();
            m_OrigTax = locTran.Tax.ToString();
            m_PreAuthAmount = tb_Amount.Text;
            m_Note = tb_Note.Text;
            m_TipAmount = tb_TipAmount.Text;
            m_OarData = locTran.OarData;
            m_Ps2000Data = locTran.Ps2000Data;
            DialogResult = true;
            this.Close();
        }
        private void b_Refund_Click(object sender, RoutedEventArgs e)
        {
            if ((0 < m_ResultCount)
             && (null != lv_Results.SelectedItems)
             && (0 < lv_Results.SelectedItems.Count))
            {
                m_DoRefund = true;
                closeDlg();
            }
            else
            {
                MessageBox.Show("No transaction selected", "INFO");
            }
        }
        private void b_TipAdjust_Click(object sender, RoutedEventArgs e)
        {
            if ((0 < m_ResultCount)
             && (null != lv_Results.SelectedItems)
             && (0 < lv_Results.SelectedItems.Count))
            {
                if (String.IsNullOrEmpty(tb_TipAmount.Text.Trim(' ')))
                {
                    MessageBox.Show("No tip amount entered", "INFO");
                }
                else
                {
                    m_DoTipAdjust = true;
                    closeDlg();
                }
            }
            else
            {
                MessageBox.Show("No transaction selected", "INFO");
            }
        }
        public bool DoRefund()
        {
            return m_DoRefund;
        }
        public bool DoTipAdjust()
        {
            return m_DoTipAdjust;
        }
        public bool DoPreAuthDelete()
        {
            return m_DoDelete;
        }
        public bool DoPrint()
        {
            return m_DoPrint;
        }
        public bool DoPreAuthIncrement()
        {
            return m_DoIncrement;
        }
        public bool DoPreAuthComplete()
        {
            return m_DoComplete;
        }
        public bool DoVoid()
        {
            return m_DoVoid;
        }
        public bool DoCustomPrint()
        {
            return m_DoCustomPrint;
        }
        public String getOrigTransId()
        {
            return m_OrigTransId;
        }
        public String getOrigDate()
        {
            return m_OrigDate;
        }
        public String getOrigTender()
        {
            return m_OrigTender;
        }
        public Transaction getPrintTran()
        {
            return m_PrintTran;
        }
        public String getOrigCardType()
        {
            return m_OrigCardType;
        }
        public String getOrigCurrency()
        {
            return m_OrigCurrency;
        }

        public String getRefundableAmount()
        {
            return m_RefundableAmount;
        }

        public String getOrigAmount()
        {
            return m_OrigAmount;
        }


        public String getOrigAmountMinusSurcharge()
        {
            return m_AmountMinusSurcharge;
        }

        public String getOrigTax()
        {
            return m_OrigTax;
        }
        public String getOrigTip()
        {
            return m_OrigTip;
        }
        public String getNote()
        {
            return m_Note;
        }
        public String getPreAuthAmount()
        {
            return m_PreAuthAmount;
        }

        public String getTipAmount()
        {
            return m_TipAmount;
        }
        public String getOarData()
        {
            return m_OarData;
        }
        public String getPs2000Data()
        {
            return m_Ps2000Data;
        }
        private void handleButtons()
        {
            if ((0 < m_ResultCount)
             && (null != lv_Results.SelectedItems)
             && (0 < lv_Results.SelectedItems.Count))
            {
                b_Print.IsEnabled = true;
                LocalTran locTran = (LocalTran)lv_Results.SelectedItems[0];
                b_Refund.IsEnabled = convertYesOrNoToBoolean(locTran.Refundable);
                b_PreAuthIncrement.IsEnabled = true;
                b_PreAuthComplete.IsEnabled = true;
                b_PreAuthDelete.IsEnabled = true;
                b_Void.IsEnabled = convertYesOrNoToBoolean(locTran.Voidable);
                b_Info.IsEnabled = true;
                b_CustomPrint.IsEnabled = true;
                b_TipAdjust.IsEnabled = convertYesOrNoToBoolean(locTran.TipAdjustable);
            }
            else
            {
                b_Print.IsEnabled = false;
                b_Refund.IsEnabled = false;
                b_PreAuthIncrement.IsEnabled = false;
                b_PreAuthComplete.IsEnabled = false;
                b_PreAuthDelete.IsEnabled = false;
                b_Void.IsEnabled = false;
                b_Info.IsEnabled = false;
                b_CustomPrint.IsEnabled = false;
                b_TipAdjust.IsEnabled = false;
            }
        }
        private void lv_Results_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            handleButtons();
        }

        private bool convertYesOrNoToBoolean(String yesOrNoValue)
        {
            yesOrNoValue = yesOrNoValue.ToLower();
            bool boolValue = false;

            switch (yesOrNoValue)
            {
                case "y": boolValue = true; break;
                case "n": boolValue = false; break;
            }

            return boolValue;
        }

        private void b_Print_Click(object sender, RoutedEventArgs e)
        {
            if ((0 < m_ResultCount)
             && (null != lv_Results.SelectedItems)
             && (0 < lv_Results.SelectedItems.Count))
            {
                m_DoPrint = true;
                closeDlg();
            }
            else
            {
                MessageBox.Show("No transaction selected", "INFO");
            }
        }

        private void b_PreAuthIncrement_Click(object sender, RoutedEventArgs e)
        {
            if ((0 < m_ResultCount)
             && (null != lv_Results.SelectedItems)
             && (0 < lv_Results.SelectedItems.Count))
            {
                m_DoIncrement = true;
                closeDlg();
            }
            else
            {
                MessageBox.Show("No transaction selected", "INFO");
            }
        }

        private void b_PreAuthDelete_Click(object sender, RoutedEventArgs e)
        {
            if ((0 < m_ResultCount)
             && (null != lv_Results.SelectedItems)
             && (0 < lv_Results.SelectedItems.Count))
            {
                m_DoDelete = true;
                closeDlg();
            }
            else
            {
                MessageBox.Show("No transaction selected", "INFO");
            }
        }

        private void b_PreAuthComplete_Click(object sender, RoutedEventArgs e)
        {
            if ((0 < m_ResultCount)
             && (null != lv_Results.SelectedItems)
             && (0 < lv_Results.SelectedItems.Count))
            {
                m_DoComplete = true;
                closeDlg();
            }
            else
            {
                MessageBox.Show("No transaction selected", "INFO");
            }
        }

        private void b_Void_Click(object sender, RoutedEventArgs e)
        {
            if ((0 < m_ResultCount)
             && (null != lv_Results.SelectedItems)
             && (0 < lv_Results.SelectedItems.Count))
            {
                m_DoVoid = true;
                closeDlg();
            }
            else
            {
                MessageBox.Show("No transaction selected", "INFO");
            }
        }

        private void b_CustomPrint_Click(object sender, RoutedEventArgs e)
        {
            m_PrintTran = ShowOrPrint(false);
            if (null != m_PrintTran)
            {
                m_DoCustomPrint = true;
                closeDlg();
            }
            else
            {
                MessageBox.Show("No transaction selected", "INFO");
            }

        }
    }
    public class LocalTran
    {
        public string Id { get; set; }
        public string TranType { get; set; }
        public string Tender { get; set; }
        public string CardEntryType { get; set; }
        public string CardType { get; set; }
        public string MaskedPan { get; set; }
        public string Date { get; set; }
        public long Amount { get; set; }
        public long AmountRefundable { get; set; }
        public long AmountRefunded { get; set; }
        public long SurchargeAmount { get; set; }
        public long SurchargeAmountRefunded { get; set; }
        public string AuthCode { get; set; }
        public string Card { get; set; }
        public string Currency { get; set; }
        public string Tip { get; set; }
        public string Tax { get; set; }
        public string Result { get; set; }
        public string Voidable { get; set; }
        public string TipAdjustable { get; set; }
        public string Refundable { get; set; }
        public string InvoiceNum { get; set; }
        public string LaneNum { get; set; }
        public string OarData { get; set; }
        public string Ps2000Data { get; set; }
        public long OtcAmount { get; set; }
        public long PrescriptionAmount { get; set; }
        public long ClinicAmount { get; set; }
        public long DentalAmount { get; set; }
        public long VisionAmount { get; set; }
        public long TransitAmount { get; set; }

        public string IvuControlNumber { get; set; }

        public string IvuMunicipalTax { get; set; }

        public string IvuStateTax { get; set; }


        //Service fee related attributes
        public string ServiceFeeTxnId { get; set; }
        public string ServiceFeeTxnApprovalCode { get; set; }
        public string ServiceFeeTxnDate { get; set; }
        public string ServiceFeeTxnAmount { get; set; }
        public string ServiceFeeTxnRefundedAmount { get; set; }
    }
}
