﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Threading;

using CWSWrapper;

/*
 * This is a very simple app which allows you to run a sale transaction and optionally print a receipt.
 */
namespace SalesApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        // replace these values with your own!
        private static readonly String MY_SERVERTYPE = "DEMO";
        private static readonly String MY_EMAIL = "hello@there.com";
        private static readonly String MY_MERCHANT = "123456";
        private static readonly String MY_USER = "somedude";
        private static readonly String MY_PIN = "ABCD";
        private static readonly String MY_VENDOR_ID = "myvendorid";
        private static readonly String MY_VENDOR_APP_NAME = "myvendorappname";
        private static readonly String MY_VENDOR_APP_VERSION = "myvendorappversion";
        
        private CWS m_CWS;
        private string m_PaymentGatewayId;
        private bool m_PrintReceipt;

        public MainWindow()
        {
            InitializeComponent();

            InitCWS();

            // default to $3.00
            tb_Amount.Text = "300";
           
            OpenPaymentGateway();
        }

		/*
		 * DoTransaction - called from the three "click" handlers.
		 * Load PaymentArgs and call StartPaymentTransaction.
		 */
        private void DoTransaction(String tt)
        {
			/* Get the amount from the UI's amount edit control */

            String myAmount = tb_Amount.Text;

			/* PaymentArgs object holds all parameters needed to start a transaction */

            PaymentArgs pa = new PaymentArgs();

			/* Create a "Money" object for the amount in USD */

            Money m = new Money(Money.CurrencyCodefromString("USD"), myAmount);

			/* Load PaymentArgs with the minimum to start a transaction */

            pa.paymentGatewayId = m_PaymentGatewayId;

            pa.baseTransactionAmount = m;
            pa.tenderType = TenderType.CARD;
            pa.transactionType = tt;

            s_Operation.Content = tt;	/* Show in UI what we are doing */

			/* Kick off a transaction.  This returns right away */

            m_CWS.StartPaymentTransaction(pa, MyNotifyCWSEvent, MyPaymentComplete);
        }

		/*
		 * Handle informational messages as they come it.  Just show them in the UI
		 */
        public void MyNotifyCWSEvent(CWSEvent cwsevent)
        {
            Dispatcher.Invoke((Action)delegate
            {
                s_Status.Content = String.Format("Event: {0}", cwsevent.statusDetails);
            });
        }

		/*
		 * Transaction is complete.  Just show results, and kick off a 
		 * print receipt if requested.
		 */
        public void MyPaymentComplete(PaymentTransactionResults ptr)
        {
            PaymentTransactionData ptdata = ptr.PaymentTransactionData;
            if (null != ptdata)
            {
                string status = String.Format("{0}", ptdata.result);
                if (ptdata.result.Equals("APPROVED"))
                {
                    if(m_PrintReceipt)
                        PrintReceipt();
                }
                else
                {
                    if (0 < ptdata.errors.Length)
                    {
                        status = String.Format("{0} - {1}", ptdata.result, ptdata.errors[0]);
                    }
                }

                Dispatcher.Invoke((Action)delegate
                {
                    s_Status.Content = status;
                });
            }
        }
		/*
		 * PrintReceipt - for the last successful transaction
		 */
        private void PrintReceipt()
        {
            PrintPaymentArgs ppa = new PrintPaymentArgs();

            ppa.paymentGatewayId = m_PaymentGatewayId;

            ppa.transactionId = null;		/* null implies receipt for last successful */

            m_CWS.StartPrintPaymentReceipt(ppa, MyNotifyCWSEvent, MyPrintComplete);
        }
		/*
		 * Print receipt is complete.
		 */
        public void MyPrintComplete(PaymentTransactionResults ptr)
        {
            PaymentTransactionData ptdata = ptr.PaymentTransactionData;
            if (null != ptdata)
            {
                Dispatcher.Invoke((Action)delegate
                {
                    s_Status.Content = String.Format("Result {0}", ptdata.result);
                });
            }
        }

		/*
		 * Click handlers - just call DoTransaction with the 
		 * different TransactionTypes.
		 */
        private void b_Sale_Click(object sender, RoutedEventArgs e)
        {
            DoTransaction(TransactionType.SALE);
        }

        private void b_Refund_Click(object sender, RoutedEventArgs e)
        {
            DoTransaction(TransactionType.STANDALONE_REFUND);
        }

        private void b_PreAuth_Click(object sender, RoutedEventArgs e)
        {
            DoTransaction(TransactionType.PRE_AUTH);
        }

		/*
		 * App is exiting.  Close the payment gateway and stop CWS.
		 */
        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            m_CWS.Stop();
            m_CWS = null;
            Thread.Sleep(100);
            base.OnClosing(e);
        }

		/*
		 * InitCWS - create an instance and call Start.
		 */
        private void InitCWS()
        {
            m_CWS = new CWS();
            m_CWS.Start("SalesLog.txt", 4);
        }

		/*
		 * OpenPaymentGateway - load up the OpenPaymentGatewayArgs object
		 * with your credentials, then call OpenPaymentGateway.
		 */
        private void OpenPaymentGateway()
        {
            OpenPaymentArgs opa = new OpenPaymentArgs();
            opa.merchantId = MY_MERCHANT;              
            opa.userId = MY_USER;
            opa.pin = MY_PIN;
            opa.vendorId = MY_VENDOR_ID;
            opa.vendorAppName = MY_VENDOR_APP_NAME;
            opa.vendorAppVersion = MY_VENDOR_APP_VERSION;
            opa.email = MY_EMAIL;
            opa.paymentGatewayEnvironment = MY_SERVERTYPE;
            opa.handleDigitalSignature = false;      

            OpenPaymentGatewayResults results = m_CWS.OpenPaymentGateway(opa, null);
            s_Operation.Content = "OpenPaymentGateway";
            if (null != results.OpenPaymentData)
            {
                m_PaymentGatewayId = results.OpenPaymentData.paymentGatewayId;
                s_Status.Content = String.Format("Result {0}", results.OpenPaymentData.result);
            }
            else
            {
                if (null != results.CWSEvents[0])
                    s_Status.Content = results.CWSEvents[0].statusDetails;
                else
                    s_Status.Content = "Failed";
            }
        }

        private void printReceiptUncheckedEvent(object sender, RoutedEventArgs e)
        {
            m_PrintReceipt = false;
        }

        private void printReceiptCheckedEvent(object sender, RoutedEventArgs e)
        {
            m_PrintReceipt = true;
        }
    }
}
